#!/usr/bin/env python2

# Rekall Memory Forensics
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Author: Michael Cohen scudette@google.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

__author__ = "Mikhail Bushkov <mbushkov@google.com>"

import argparse
import os
import sys
import traceback
import webbrowser

from rekall import config
from rekall import plugin
from rekall import testlib

from rekall.plugins.renderers import data_export

from flask import Blueprint
from flask import jsonify
from flask import request

from manuskript import server as manuskript_server
from manuskript import plugin as manuskript_plugin
from manuskript import plugins as manuskript_plugins


try:
    STATIC_PATH = os.path.join(sys._MEIPASS, "webconsole") # pylint: disable=protected-access
except AttributeError:
    STATIC_PATH = os.path.join(os.path.dirname(__file__), "webconsole")



class WebConsoleRenderer(data_export.DataExportRenderer):
    # This renderer is private to the web console.
    name = None

    def __init__(self, message_queue, *args, **kwargs):
        self.message_queue = message_queue
        super(WebConsoleRenderer, self).__init__(*args, **kwargs)

    def write_data_stream(self):
        if self.data:
            self.message_queue.extend(self.data)


class WebConsoleObjectRenderer(data_export.NativeDataExportObjectRenderer):
    renders_type = "object"
    renderers = ["WebConsoleRenderer"]

    def _GetDelegateObjectRenderer(self, item):
        return self.FromEncoded(item, "DataExportRenderer")(
            renderer=self.renderer)

    def EncodeToJsonSafe(self, item, **options):
        return self._GetDelegateObjectRenderer(item).EncodeToJsonSafe(
            item, **options)

    def DecodeFromJsonSafe(self, value, options):
        return self._GetDelegateObjectRenderer(value).DecodeFromJsonSafe(
            value, options)


class RekallPythonCall(manuskript_plugins.PythonCall):
    """PythonCall extension that inserts Rekall session into local context."""

    @classmethod
    def UpdatePythonShell(cls, app, shell):
        super(RekallPythonCall, cls).UpdatePythonShell(app, shell)

        rekall_session = app.config["rekall_session"]
        shell.local_context = rekall_session._locals # pylint: disable=protected-access
        shell.global_context["rekall_session"] = rekall_session


class FakeParser(object):
    """Fake parser used to reflect on plugins' arguments."""

    def __init__(self):
        self.arguments = []

    def add_argument(self, name, *unused_args, **kwargs):
        # Remove starting dashes.
        while name.startswith("-"):
            name = name[1:]

        argument_def = {}
        argument_def["name"] = name

        argument_def["required"] = kwargs.get("required", False)

        if "help" in kwargs:
            argument_def["help"] = kwargs["help"]

        if "nargs" in kwargs:
            argument_def["nargs"] = kwargs["nargs"]

        if "choices" in kwargs:
            if hasattr(kwargs["choices"], "keys"):
                argument_def["choices"] = kwargs["choices"].keys()
            else:
                argument_def["choices"] = list(kwargs["choices"])

        passed_default = kwargs.get("default", None)
        if (passed_default is None and
            "choices" in argument_def and argument_def["choices"]):
            passed_default = argument_def["choices"][0]

        arg_type = "string"

        # Try to guess argument type by the type of a default value.
        if passed_default:
            argument_def["default"] = passed_default
            if isinstance(passed_default, bool):
                arg_type = "bool"
            elif isinstance(passed_default, int):
                arg_type = "int"
        # If we can't guess by default value, inspect parser's action.
        elif "action" in kwargs:
            action = kwargs["action"]
            if action is config.IntParser:
                arg_type = "int"
            elif action is config.ArrayIntParser:
                arg_type = "int"
            elif action == "store_true":
                arg_type = "bool"

        argument_def["type"] = arg_type
        self.arguments.append(argument_def)


class RekallWebConsole(manuskript_plugin.Plugin):

    ANGULAR_MODULE = "rekall.webconsole"

    JS_FILES = [
        "/rekall-webconsole/jsonrenderer-service.js",
        "/rekall-webconsole/pluginregistry-service.js",
        "/rekall-webconsole/pluginarguments-directive.js",
        "/rekall-webconsole/runplugin-controller.js",
        "/rekall-webconsole/runplugin.js",
        "/rekall-webconsole/webconsole.js",
        "/rekall-webconsole/third_party/python-format.js"
        ]

    CSS_FILES = [
        "/rekall-webconsole/runplugin.css"
        ]

    @classmethod
    def PlugIntoApp(cls, app):
        super(RekallWebConsole, cls).PlugIntoApp(app)

        # Use blueprint as an easy way to serve static files.
        bp = Blueprint('rekall-webconsole', __name__,
                       static_url_path="/rekall-webconsole",
                       static_folder=STATIC_PATH)
        @bp.after_request
        def add_header(response):  # pylint: disable=unused-variable
            response.headers['Cache-Control'] = 'no-cache, no-store'
            return response
        app.register_blueprint(bp)

        @app.route("/rekall/plugins/all")
        def list_all_plugins():   # pylint: disable=unused-variable
            session = app.config['rekall_session']
            plugins = plugin.Command.GetActiveClasses(session)

            result = {}
            for plugin_cls in plugins:
                plugin_def = {}

                plugin_def["name"] = getattr(plugin_cls, "name", None)

                if not plugin_def["name"]:
                    continue

                plugin_def["description"] = plugin_cls.__doc__

                parser = FakeParser()
                plugin_cls.args(parser)
                plugin_def["arguments"] = parser.arguments

                result[plugin_def["name"]] = plugin_def

            return jsonify({"data": result})

        @app.route("/rekall/runplugin", methods=["POST"])
        def run_plugin():   # pylint: disable=unused-variable
            rekall_session = app.config["rekall_session"]
            source = request.get_json()["source"]

            message_queue = []

            data_renderer = WebConsoleRenderer(
                message_queue, session=rekall_session)

            with data_renderer.start():
                try:
                    args = _parse_args(source["arguments"], source["plugin"])

                    rekall_session.RunPlugin(
                        source["plugin"]["name"], renderer=data_renderer,
                        **args)
                except Exception:
                    data_renderer.report_error(traceback.format_exc())

            response = jsonify(dict(json_output=message_queue))
            return response



def _parse_args(args, plugin_metadata):
    """Parses the args returned by the form.

    The WebConsole form returns everything as strings, however Rekall's plugins
    require some parameters as integers etc.

    This function converts the strings returned by the browser into the required
    types.
    """
    result = {}
    for k, v in args.items():
        if v == "":
            continue

        for arg in plugin_metadata["arguments"]:
            if arg["name"] == k:
                k = k.replace("-", "_")

                if arg["type"] == "int":
                    result[k] = int(v)
                else:
                    result[k] = v

                break


    return result


class WebConsole(plugin.Command):
    """Launch the web-based console notebook."""

    __name = "webconsole"

    @classmethod
    def args(cls, parser):
        super(WebConsole, cls).args(parser)

        parser.add_argument("--host", default="localhost",
                            help="Host for the web console to use.")

        parser.add_argument("--port", default=0, type=int,
                            help="Port for the web console to use.")

        parser.add_argument("--debug", default=False, action='store_true',
                            help="Start in the debug mode (will monitor "
                            "changes in the resources and reload them as "
                            "needed.")

        parser.add_argument("--no_browser", default=False,
                            action='store_true',
                            help="Don't open webconsole in the default browser.")

    def __init__(self, host="localhost", port=0, debug=False,
                 no_browser=False, **kwargs):
        super(WebConsole, self).__init__(**kwargs)
        self.host = host
        self.port = port
        self.debug = debug
        self.no_browser = no_browser

    def server_post_activate_callback(self, server):
        # Update the port number, because the server may have launched on a
        # random port.
        self.port = server.server_address[1]
        if not self.no_browser:
            webbrowser.open("http://%s:%d" % (self.host, self.port))
        else:
            sys.stderr.write(
                "\nSupressing web browser (--no_browser flag). "
                "Server running at http://%s:%d\n" % (self.host, self.port))

    def render(self, renderer):
        renderer.format("Starting Manuskript web console.")
        renderer.format("Press Ctrl-c to return to the interactive shell.")

        with self.session:
            self.session.SetParameter("renderer", "DataExportRenderer")

        manuskript_server.RunServer(
            host=self.host, port=self.port, debug=self.debug,
            plugins=[manuskript_plugins.PlainText,
                     manuskript_plugins.Markdown,
                     RekallWebConsole, RekallPythonCall],
            config=dict(rekall_session=self.session),
            post_activate_callback=self.server_post_activate_callback)


class TestWebConsole(testlib.DisabledTest):
    """Disable the test for this command to avoid bringing up the notebook."""
    PARAMETERS = dict(commandline="webconsole")



# Rekall Memory Forensics
#
# Copyright 2014 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""
Collectors for sockets and other network-related things.
"""
__author__ = "Adam Sindelar <adamsh@google.com>"

from rekall import components
from rekall import identity as id


def ParseSocket(socket):
    if socket.addressing_family in ("AF_INET", "AF_INET6"):
        connection = components.Connection(
            addressing_family=socket.addressing_family,
            state=socket.tcp_state,
            protocols=(
                "IPv4" if socket.addressing_family == "AF_INET" else "IPv6",
                socket.l4_protocol,
            ),
            src_addr=socket.src_addr,
            src_bind=socket.src_port,
            dst_addr=socket.dst_addr,
            dst_bind=socket.dst_port,
        )

        named = components.Named(
            name="%s/%d -> %s/%d" % (
                connection.src_addr, connection.src_bind,
                connection.dst_addr, connection.dst_bind),
            kind="%s socket" % connection.protocols[0],
        )
    elif socket.addressing_family == "AF_UNIX":
        connection = components.Connection(
            addressing_family="AF_UNIX",
            state=None,
            src_addr="0x%x" % int(socket.so_pcb),
            dst_addr="0x%x" % int(socket.unp_conn),
            protocols=("Unix", socket.unix_type),
            src_bind=socket.get_socketinfo_attr("unsi_addr"),
            dst_bind=None,
        )

        named = components.Named(
            name="%s->%s" % (connection.src_addr, connection.dst_addr),
            kind="%s socket" % connection.protocols[0],
        )
    else:
        connection = components.Connection(
            addressing_family=socket.addressing_family,
            state=None,
            src_addr=None,
            src_bind=None,
            dst_addr=None,
            dst_bind=None,
            protocols=[socket.addressing_family],
        )

        named = components.Named(
            name=None,
            kind="%s socket" % connection.addressing_family,
        )

    return connection, named


def ParseVnode(vnode):
    file = components.File(
        full_path=vnode.full_path,
    )

    named = components.Named(
        name=vnode.full_path,
        kind="Reg. File",
    )

    return file, named


def UnixSocketCollector(profile):
    """Walks the global unpcb lists and returns just the sockets.

    Yields:
      DarwinUnixSocket entities.

    See here:
      https://github.com/opensource-apple/xnu/blob/10.9/bsd/kern/uipc_usrreq.c#L121
    """
    for head_const in ["_unp_dhead", "_unp_shead"]:
        lhead = profile.get_constant_object(
            head_const,
            target="unp_head")

        for unp in lhead.lh_first.walk_list("unp_link.le_next"):
            connection, named = ParseSocket(unp.unp_socket)
            yield id.BaseObjectIdentity(unp.unp_socket), (connection, named)


def FileprocHandleCollector(profile):
    for process in profile.session.entities.find_by_component("Process"):
        base_obj = process.components.MemoryObject.base_object

        for fd, fileproc, flags in base_obj.get_open_files():
            data = fileproc.autocast_fg_data()

            # If the above returns a None then that's a serious error. Fail
            # early.
            assert data

            # Yield both the handle and a stub of the resource. We don't want to
            # parse the resource-specific components yet, because they're not
            # really needed if we're just looking for handles, but at the same
            # time, we don't want to have to go back and look for all the
            # fg_data structs again.
            resource_id = id.BaseObjectIdentity(data)
            handle_id = id.BaseObjectIdentity(fileproc)

            handle = components.Handle(
                process=process.identity,
                fd=fd,
                flags=flags,
                resource=resource_id,
            )

            handle_mem_obj = components.MemoryObject(
                base_object=fileproc,
                type="fileproc",
            )

            yield handle_id, (handle, handle_mem_obj)

            # All we need of the resource is the overlay struct, which we can
            # parse later.
            resource_mem_obj = components.MemoryObject(
                base_object=data,
                type=type(data).__name__,
            )

            yield resource_id, (resource_mem_obj,)


def HandleSocketCollector(profile):
    # Indexing commented out because, right now, it actually makes things
    # slower by about 300 ms on the first run (subsequent runs are obviously
    # much faster). TODO: Try and optimize indexing, maybe by doing it in
    # batches, or deferring it.
    # profile.session.entities.add_attribute_lookup("MemoryObject", "type")

    # Make sure all the handles are collected.
    profile.session.entities.run_collector(FileprocHandleCollector)

    # We just need the base objects for sockets.
    for entity in profile.session.entities.find_by_attribute(
        "MemoryObject.type", "socket"):
        connection, named = ParseSocket(entity["MemoryObject.base_object"])

        # The original entity /is/ the socket, so we just reuse the identity.
        yield entity.identity, (connection, named)


def HandleVnodeCollector(profile):
    # Collect all the handles.
    profile.session.entities.run_collector(FileprocHandleCollector)

    # All we need are vnodes.
    for entity in profile.session.entities.find_by_attribute(
        "MemoryObject.type", "vnode"):
        file, named = ParseVnode(entity["MemoryObject.base_object"])

        yield entity.identity, (file, named)


def ParseNetworkInterface(interface):
    yield components.Named(
        name="%s%d" % (
            interface.if_name.deref(),
            interface.if_unit,
        ),
        kind="Network Interface",
    )

    yield components.NetworkInterface(
        addresses=[
            (x.ifa_addr.sa_family, x.ifa_addr.deref())
            for x
            in interface.if_addrhead.tqh_first.walk_list("ifa_link.tqe_next")
        ],
    )


def NetworkInterfaces(profile):
    """Walks the global list of interfaces.

    The head of the list of network interfaces is a kernel global [1].
    The struct we use [2] is just the public part of the data [3]. Addresses
    are related to an interface in a N:1 relationship [4]. AF-specific data
    is a normal sockaddr struct.

    Yields:
      Network interfaces.

    References:
      1:
      https://github.com/opensource-apple/xnu/blob/10.9/bsd/net/dlil.c#L254
      2:
      https://github.com/opensource-apple/xnu/blob/10.9/bsd/net/if_var.h#L528
      3:
      https://github.com/opensource-apple/xnu/blob/10.9/bsd/net/dlil.c#L188
      4:
      https://github.com/opensource-apple/xnu/blob/10.9/bsd/net/if_var.h#L816

    """
    ifnet_head = profile.get_constant_object(
        "_dlil_ifnet_head",
        target="Pointer",
        target_args=dict(
            target="ifnet"
        )
    )

    for interface in ifnet_head.walk_list("if_link.tqe_next"):
        yield interface, ParseNetworkInterface(interface)




#!/usr/bin/python

# Rekall Memory Forensics
# Copyright (C) 2012 Michael Cohen <scudette@gmail.com>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
"""This module manages the command line parsing logic."""

__author__ = "Michael Cohen <scudette@gmail.com>"

import argparse
import logging
import os
import sys
import zipfile

from rekall import config
from rekall import constants
from rekall import plugin


config.DeclareOption("--plugin", default=[], nargs="+",
                     help="Load user provided plugin bundle.")

config.DeclareOption(
    "-h", "--help", default=False, action="store_true",
    help="Show help about global paramters.")


class MockArgParser(object):
    def add_argument(self, short_flag="", long_flag="", dest="", **_):
        if short_flag.startswith("--"):
            flag = short_flag
        elif long_flag.startswith("--"):
            flag = long_flag
        elif dest:
            flag = dest
        else:
            flag = short_flag

        # This function will be called by the args() class method, and we just
        # keep track of the args this module defines.
        arg_name = flag.strip("-").replace("-", "_")

        self.args[arg_name] = None

    def build_args_dict(self, cls, namespace):
        """Build a dict suitable for **kwargs from the namespace."""
        self.args = {}

        # Discover all the args this module uses.
        cls.args(self)

        for key in self.args:
            value = getattr(namespace, key, None)
            if value is not None:
                self.args[key] = value

        return self.args


class RekallHelpFormatter(argparse.RawDescriptionHelpFormatter):
    def add_argument(self, action):
        # Allow us to suppress an arg from the --help output for those options
        # which do not make sense on the command line.
        if action.dest != "SUPPRESS":
            super(RekallHelpFormatter, self).add_argument(action)


class RekallArgParser(argparse.ArgumentParser):
    ignore_errors = False

    def __init__(self, **kwargs):
        kwargs["formatter_class"] = RekallHelpFormatter
        super(RekallArgParser, self).__init__(**kwargs)

    def error(self, message):
        if self.ignore_errors:
            return

        # We trap this error especially since we launch the volshell.
        if message == "too few arguments":
            return

        super(RekallArgParser, self).error(message)

    def parse_known_args(self, args=None, namespace=None, force=False, **_):
        self.ignore_errors = force

        result = super(RekallArgParser, self).parse_known_args(
            args=args, namespace=namespace)

        return result

    def print_help(self, file=None):
        if self.ignore_errors:
            return

        return super(RekallArgParser, self).print_help(file=file)

    def exit(self, *args, **kwargs):
        if self.ignore_errors:
            return

        return super(RekallArgParser, self).exit(*args, **kwargs)


def LoadPlugins(paths=None):
    PYTHON_EXTENSIONS = [".py", ".pyo", ".pyc"]

    for path in paths:
        if not os.access(path, os.R_OK):
            logging.error("Unable to find %s", path)
            continue

        path = os.path.abspath(path)
        directory, filename = os.path.split(path)
        module_name, ext = os.path.splitext(filename)

        # Its a python file.
        if ext in PYTHON_EXTENSIONS:
            # Make sure python can find the file.
            sys.path.insert(0, directory)

            try:
                logging.info("Loading user plugin %s", path)
                __import__(module_name)
            except Exception, e:
                logging.error("Error loading user plugin %s: %s", path, e)
            finally:
                sys.path.pop(0)

        elif ext == ".zip":
            zfile = zipfile.ZipFile(path)

            # Make sure python can find the file.
            sys.path.insert(0, path)
            try:
                logging.info("Loading user plugin archive %s", path)
                for name in zfile.namelist():
                    # Change from filename to python package name.
                    module_name, ext = os.path.splitext(name)
                    if ext in PYTHON_EXTENSIONS:
                        module_name = module_name.replace("/", ".").replace(
                            "\\", ".")

                        try:
                            __import__(module_name.strip("\\/"))
                        except Exception as e:
                            logging.error("Error loading user plugin %s: %s",
                                          path, e)

            finally:
                sys.path.pop(0)

        else:
            logging.error("Plugin %s has incorrect extension.", path)


def _TruncateARGV(argv):
    """Truncate the argv list at the first sign of a plugin name.

    At this stage we do not know which module is valid, or its options. The
    syntax of the command line is:

    rekal -x -y -z plugin_name -a -b -c

    Where -x -y -z are global options, and -a -b -c are plugin option.  We only
    want to parse up to the plugin name.
    """
    short_argv = [argv[0]]
    for item in argv[1:]:
        for plugin_cls in plugin.Command.classes.values():
            if plugin_cls.name == item:
                return short_argv

        short_argv.append(item)

    return short_argv

def LoadProfileIntoSession(parser, argv, user_session):
    # Figure out the profile.
    argv = argv or sys.argv
    known_args, _ = parser.parse_known_args(args=_TruncateARGV(argv))

    with user_session.state as state:
        config.MergeConfigOptions(state)

        for arg, value in vars(known_args).items():
            state.Set(arg, value)

        # Enforce the appropriate logging level if user supplies the --verbose
        # or --quiet command line flags.
        verbose_flag = getattr(known_args, "verbose", None)
        quiet_flag = getattr(known_args, "quiet", None)

        if verbose_flag and quiet_flag:
            raise ValueError("Cannot set both --verbose and --quiet!")

        if verbose_flag:
            state.Set("logging", "debug")
        elif quiet_flag:
            state.Set("logging", "critical")
        else:
            state.Set("logging", "warn")

    # Now load the third party user plugins. These may introduce additional
    # plugins with args.
    LoadPlugins(user_session.state.plugin)

    session_filename = getattr(known_args, "session_filename", None)
    if session_filename:
        try:
            user_session.LoadFromFile(session_filename)

            # Set the command line args once again, in case they override
            # something in the stored session.
            with user_session.state as state:
                for arg, value in known_args.__dict__.items():
                    state.Set(arg, value)

        except IOError:
            pass


def parse_args(argv=None, user_session=None):
    """Parse the args from the command line argv."""
    parser = RekallArgParser(
        description=constants.BANNER,
        conflict_handler='resolve',
        add_help=False,
        epilog="When no module is provided, drops into interactive mode",
        formatter_class=RekallHelpFormatter)

    config.RegisterArgParser(parser)

    # First load the profile to enable the module selection (which depends on
    # the profile).
    LoadProfileIntoSession(parser, argv, user_session)

    # Add module specific args.
    subparsers = parser.add_subparsers(
        description="The following plugins can be selected.",
        metavar="Plugin",
        )

    parsers = {}

    # Add module specific parser for each module.
    classes = []
    for cls in plugin.Command.classes.values():
        if (cls.name and cls.is_active(user_session) and not
            cls.interactive):
            classes.append(cls)

    for cls in sorted(classes, key=lambda x: x.name):
        docstring = cls.__doc__ or " No Docs "
        doc = docstring.splitlines()[0] or " No Docs "
        name = cls.name
        try:
            module_parser = parsers[name]
        except KeyError:
            parsers[name] = module_parser = subparsers.add_parser(
                cls.name, help=doc, description=docstring)

            cls.args(module_parser)
            module_parser.set_defaults(module=cls.name)

    # Parse the final command line.
    result = parser.parse_args(argv)

    # We handle help especially since we want to enumerate all plugins.
    if getattr(result, "help", None):
        parser.print_help()
        sys.exit(-1)

    return result



# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

__author__ = "Michael Cohen <scudette@google.com>"

from rekall.plugins.darwin import common
from rekall.plugins.darwin import lsof


class DarwinArp(common.DarwinPlugin):
    """Show information about arp tables."""

    __name = "arp"

    def render(self, renderer):
        renderer.table_header(
            [("IP Addr", "ip_addr", "20"),
             ("MAC Addr", "mac", "18"),
             ("Interface", "interface", "9"),
             ("Sent", "sent", "8"),
             ("Recv", "recv", "8"),
             ("Time", "timestamp", "24"),
             ("Expires", "expires", "8"),
             ("Delta", "delta", "8"),
             ])

        arp_cache = self.profile.get_constant_object(
            "_llinfo_arp",
            target="Pointer",
            target_args=dict(
                target="llinfo_arp"
                )
            )

        while arp_cache:
            entry = arp_cache.la_rt

            renderer.table_row(
                entry.source_ip,
                entry.dest_ip,
                entry.name,
                entry.sent,
                entry.rx,
                entry.base_calendartime,
                entry.rt_expire,
                entry.delta
                )

            arp_cache = arp_cache.la_le.le_next


class DarwinRoute(common.DarwinPlugin):
    """Show routing table."""

    __name = "route"

    RNF_ROOT = 2

    def rn_walk_tree(self, h):
        """Walks the radix tree starting from the header h.

        This function is taken from
        xnu-2422.1.72/bsd/net/radix.c: rn_walk_tree()

        Which is why it does not conform to the style guide.

        Note too that the darwin source code abuses C macros:

        #define rn_dupedkey     rn_u.rn_leaf.rn_Dupedkey
        #define rn_key          rn_u.rn_leaf.rn_Key
        #define rn_mask         rn_u.rn_leaf.rn_Mask
        #define rn_offset       rn_u.rn_node.rn_Off
        #define rn_left         rn_u.rn_node.rn_L
        #define rn_right        rn_u.rn_node.rn_R

        And then the original code does:
        rn = rn.rn_left

        So we replace these below.
        """
        rn = h.rnh_treetop

        seen = set()

        # First time through node, go left */
        while rn.rn_bit >= 0:
            rn = rn.rn_u.rn_node.rn_L

        while rn and rn not in seen:
            base = rn

            seen.add(rn)

            # If at right child go back up, otherwise, go right
            while (rn.rn_parent.rn_u.rn_node.rn_R == rn and
                   not rn.rn_flags & self.RNF_ROOT):
                rn = rn.rn_parent

            # Find the next *leaf* to start from
            rn = rn.rn_parent.rn_u.rn_node.rn_R
            while rn.rn_bit >= 0:
                rn = rn.rn_u.rn_node.rn_L

            next = rn

            # Process leaves
            while True:
                rn = base
                if not rn:
                    break

                base = rn.rn_u.rn_leaf.rn_Dupedkey
                if not rn.rn_flags & self.RNF_ROOT:
                    yield rn

            rn = next
            if rn.rn_flags & self.RNF_ROOT:
                return

    def render(self, renderer):
        renderer.table_header(
            [("Source IP", "source", "20"),
             ("Dest IP", "dest", "20"),
             ("Interface", "interface", "9"),
             ("Sent", "sent", "8"),
             ("Recv", "recv", "8"),
             ("Time", "timestamp", "24"),
             ("Expires", "expires", "8"),
             ("Delta", "delta", "8"),
             ])

        route_tables = self.profile.get_constant_object(
            "_rt_tables",
            target="Array",
            target_args=dict(
                count=32,
                target="Pointer",
                target_args=dict(
                    target="radix_node_head"
                    )
                )
            )

        for node in self.rn_walk_tree(route_tables[2]):
            rentry = node.dereference_as("rtentry")

            renderer.table_row(
                rentry.source_ip,
                rentry.dest_ip,
                rentry.name,
                rentry.sent, rentry.rx,
                rentry.base_calendartime,
                rentry.rt_expire,
                rentry.delta)


class DarwinIFConfig(common.DarwinPlugin):
    """List network interface information."""

    __name = "ifconfig"

    def render(self, renderer):
        renderer.table_header([("Interface", "interface", "10"),
                               ("Address", "address", "20")])

        ifnet_head = self.profile.get_constant_object(
            "_dlil_ifnet_head",
            target="Pointer",
            target_args=dict(
                target="ifnet"
                )
            )

        for interface in ifnet_head.walk_list("if_link.tqe_next"):
            for address in interface.if_addrhead.tqh_first.walk_list(
                "ifa_link.tqe_next"):
                name = "%s%d" % (interface.if_name.deref(),
                                      interface.if_unit)

                renderer.table_row(
                    name, address.ifa_addr.deref())


class DarwinIPFilters(common.DarwinPlugin):
    """Check IP Filters for hooks."""

    __name = "ip_filters"

    def render(self, renderer):
        renderer.table_header([
                ("Context", "context", "10"),
                ("Filter", "filter", "16"),
                ("Handler", "handler", "[addrpad]"),
                ("Symbol", "symbol", "20")])

        lsmod = self.session.plugins.lsmod(session=self.session)

        for list_name in ["_ipv4_filters", "_ipv6_filters"]:
            filter_list = self.profile.get_constant_object(
                list_name, target="ipfilter_list")

            for item in filter_list.tqh_first.walk_list("ipf_link.tqe_next"):
                filter = item.ipf_filter
                name = filter.name.deref()
                handler = filter.ipf_input.deref()
                renderer.table_row("INPUT", name, handler,
                                   lsmod.ResolveSymbolName(handler))

                handler = filter.ipf_output.deref()
                renderer.table_row("OUTPUT", name, handler,
                                   lsmod.ResolveSymbolName(handler))

                handler = filter.ipf_detach.deref()
                renderer.table_row("DETACH", name, handler,
                                   lsmod.ResolveSymbolName(handler))


class DarwinNetstat(lsof.DarwinLsof):
    """List per process network connections."""

    __name = "netstat"

    def sockets(self):
        for open_file in self.lsof():
            if open_file["fileproc"].fg_type != "DTYPE_SOCKET":
                continue

            sock = open_file["fileproc"].autocast_fg_data()
            if sock.addressing_family in ["AF_INET", "AF_INET6", "AF_UNIX"]:
                yield open_file

    def render(self, renderer):
        inet_by_proto = {}
        unix_socks = []

        # Group sockets by protocol/addressing family.
        for open_file in self.sockets():
            sock = open_file["fileproc"].autocast_fg_data()
            proto = sock.l4_protocol

            if proto:
                inet_by_proto.setdefault(proto, []).append((sock, open_file))
            else:
                unix_socks.append((sock, open_file))

        # Render all inet protos in alphabetical order (like netstat).
        renderer.section("Active Internet connections")
        renderer.table_header([
            ("Proto", "proto", "14"),
            ("SAddr", "saddr", "30"),
            ("SPort", "sport", "8"),
            ("DAddr", "daddr", "30"),
            ("DPort", "dport", "5"),
            ("State", "state", "15"),
            ("Pid", "pid", "8"),
            ("Comm", "comm", "20"),
        ])

        for proto, sockets in sorted(inet_by_proto.iteritems()):
            for sock, open_file in sockets:
                renderer.table_row(
                    proto,
                    sock.src_addr,
                    sock.src_port,
                    sock.dst_addr,
                    sock.dst_port,
                    sock.tcp_state,
                    open_file["proc"].pid,
                    open_file["proc"].p_comm,
                )

        # Render the UNIX sockets.
        renderer.section("Active UNIX domain sockets")
        renderer.table_header([
            ("Address", "address", "14"),
            ("Conn", "conn", "14"),
            ("Type", "type", "10"),
            ("Vnode", "vnode", "14"),
            ("Path", "path", "60"),
            ("Pid", "pid", "8"),
            ("Comm", "comm", "20"),
        ])

        for sock, open_file in unix_socks:
            renderer.table_row(
                "0x%x" % int(sock.so_pcb),
                "0x%x" % int(sock.unp_conn),
                sock.human_type,
                "0x%x" % int(sock.vnode),
                sock.human_name,
                open_file["proc"].pid,
                open_file["proc"].p_comm,
            )



# Rekall Memory Forensics
#
# Copyright 2014 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@google.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""This module generates types automatically by disassembling code.

Generally Rekall prefers to use publicly available debugging information for
building profiles for the different operating systems supported. However, in
many cases, the symbols we need are not publicly available.

We can, in theory distribute hand written profiles, for each OS version but this
has a number of disadvantages:

- It is very time intensive to prepare hand written profiles for each version of
  the operating system.

- It is not possible for us to test all possible versions (The Rekall profile
  repository has currently hundreds of windows kernel builds - we would need to
  maintain the profiles for each of these versions, and add new profiles each
  time a hotfix is released.

- We also need to document how we arrive at these offsets in order for others to
  verify our finding. It is important for a forensic application to be as
  transparent as possible without "magic numbers" in code.

An additional requirement is that the disassembly process be data driven. This
way we can update the exact algorithm within the Rekall profile repository if an
error or bug is discovered without needing to update all current users of the
tool.


How does it work?
=================

The DynamicProfile profile is loaded from the profile repository as normal. Like
in a regular profile, the profile is defined by a json object. A DynamicProfile
however, contains an additional section $DYNAMIC_STRUCT, which will be compiled
into overlays.


{
 "$METADATA": {
   .....
 }

 "$DYNAMIC_STRUCT": {
   "_TCP_LISTENER": {
     "Owner": [
       ["Disassembler", {
         "start": "tcpip.sys!_TcpCovetNetBufferList@20",
         "rule": [
           "MOV EAX, [ESI+$out]",
           "TEST EAX, EAX",
           "PUSH EAX",
           "CALL DWORD *__imp__PsGetProcessId",
         ],
         target="unsigned int"
       }]
      ]
    }
  }
}
"""

__author__ = "Michael Cohen <scudette@gmail.com>"
import logging
import re

from rekall import registry
from rekall import obj
from rekall import utils


class DynamicParser(object):
    """A dynamic profile processor base class."""

    __metaclass__ = registry.MetaclassRegistry

    def calculate(self, session):
        """Returns the expected value or a NoneObject."""
        _ = session
        return obj.NoneObject("No value found")


DISASSEMBLER_CACHE = utils.FastStore()


class Disassembler(DynamicParser):
    """A constant generator deriving values based on the disassembler."""

    def __init__(self, start=None, end=None, length=100,
                 rules=None):
        """Derive a value from disassembly.

        Args:
          start: Where to start disassembly (Usually a symbol name).
          end: Where to stop disassembly.

          length: If end is not specified, we disassemble at most this many
            bytes.

          rules: A list of rules (see above).
        """
        self.rules = self.CompileRule(rules)
        self.start = start
        self.length = length
        self.end = end
        self.cached_value = None

    def __str__(self):
        return "Disassemble %s" % self.start

    def CompileRule(self, rule):
        """Convert the rule into a regular expression.

        Rules are a list of patterns. Each pattern corresponds to a single
        instruction. There can be an arbitrary number of instructions between
        each rule.

        Output is captured using $out (this can only be specified once). Wild
        cards are denoted by *. Wildcards only apply across a single instruction
        (and comment). The following is an example of a rule:

        MOV EAX, [ESI+$out]
        TEST EAX, EAX
        PUSH EAX
        CALL DWORD *__imp__PsGetProcessId
        """
        # Sanitize all regular expression chars in the rule.
        result = []
        for line in rule:
            line = re.sub(r"([()\[\]\+])", r"\\\1", line)
            line = re.sub(r"\*", r".+?", line)
            line = re.sub(r"\$out", r"(?P<out>[^ \[\]+-]+?)", line)
            result.append(re.compile(line, re.S | re.M))

        return result

    def calculate(self, session):
        if self.cached_value is not None:
            return self.cached_value

        self.cached_value = self._calculate(session)
        return self.cached_value

    def _FindRuleIndex(self, line):
        for i, rule in enumerate(self.rules):
            # At every line we check if the current rule can be matched - if
            # it can then it is a better match.
            m = rule.search(line)
            if m:
                out = m.groupdict().get("out")

                return i, out

        return None, None

    def _SmallestVector(self, hits):
        """Find the vector with the smallest distance."""
        distance = 1e6
        result = []
        for vector, vector_distance in self.CalcDistance(hits):
            if vector_distance < distance:
                result = vector
                distance = vector_distance

        return result

    def CalcDistance(self, hits):
        n_hits = len(hits)

        while hits[0]:
            for rule in range(1, n_hits):
                while hits[rule] and hits[rule][0] <= hits[rule - 1][0]:
                    hits[rule].pop(0)

                if not hits[rule]:
                    return

            vector = [hits[i][0] for i in hits]
            distance = hits[n_hits - 1][0] - hits[0][0]
            yield vector, distance

            hits[0].pop(0)

    def _calculate(self, session):
        # Try to cache disassembly to speed things up.
        try:
            disassembly = DISASSEMBLER_CACHE.Get(
                (self.start, self.length, self.end))
        except KeyError:
            disassembly = unicode(session.plugins.dis(
                    offset=self.start,
                    length=self.length, end=self.end))

            DISASSEMBLER_CACHE.Put(
                (self.start, self.length, self.end), disassembly)

        hits = {}
        outs = {}

        for hit, line in enumerate(disassembly.splitlines()):
            rule_idx, out = self._FindRuleIndex(line)

            if rule_idx is None:
                continue

            hits.setdefault(rule_idx, []).append(hit)
            if out:
                outs[hit] = out

        for item in self._SmallestVector(hits):
            out = outs.get(item)
            if out:
                return int(out, 0)

        return 0


class DynamicProfile(obj.Profile):
    """A Dynamic profile which parses its overlays from $DYNAMIC_STRUCT."""




def GenerateOverlay(dynamic_definition):
    """Parse the definition and generate an overlay from it."""
    overlay = {}
    for type_name, definition in dynamic_definition.items():
        type_overlay = {}
        overlay[type_name] = [None, type_overlay]

        for field_name, attempts in definition.items():
            parsers = []
            for (parser_name, kwargs) in attempts:
                target = kwargs.pop("target", None)
                target_args = kwargs.pop("target_args", {})

                parsers.append(DynamicParser.classes.get(parser_name)(
                        **kwargs))

            # Make the offset a callable
            # Bind parameters in lambda:
            # pylint: disable=dangerous-default-value
            def offset_cb(x, parsers=parsers, field_name=field_name):
                for p in parsers:
                    result = p.calculate(x.obj_session)
                    if result:
                        return result
                    else:
                        logging.debug(
                            "Unable to find %s.%s via %s", x.obj_name,
                            field_name, p)

                return 0

            type_overlay[field_name] = [offset_cb, [target, target_args]]

    return overlay



# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

"""
This module defines all the components that Rekall knows about.

A component is a collection of properties that relate to an entity, which is
an encapsulated notion of identity. In Rekall, components are basically just
named tuples which we store in a big hashtable, indexed by the entity they
relate to.

See:
  http://en.wikipedia.org/wiki/Entity_component_system
"""
__author__ = "Adam Sindelar <adamsh@google.com>"

import collections

from rekall import superposition


COMPONENTS = [
    "Entity",  # Has to be first!
    "Named",
    "MemoryObject",
    "NetworkInterface",
    "Process",
    "Thread",
    "Connection",
    "Handle",
    "File",
    "User",
    "Group",
#    "Timer",
#    "Route",
#    "LoginSession",
#    "KernelModule",
#    "Mount",
#    "AllocationZone",
]


class ComponentTuple(collections.namedtuple("ComponentTuple", COMPONENTS)):
    """Has a property named after every component defined in this module."""

    def union(self, other):
        components = [
            superposition.SuperpositionMergeNamedTuples(
                getattr(self, component),
                getattr(other, component),
            )
            for component in COMPONENTS[1:]  # Skip Entity.
        ]

        # Entity is merged using slightly simpler rules.
        entity = Entity(
            identity=self.Entity.identity | other.Entity.identity,
            collectors=self.Entity.collectors | other.Entity.collectors,
        )

        return ComponentTuple(
            entity,
            *components
        )

    def __or__(self, other):
        return self.union(other)


__EmptyComponentTuple = ComponentTuple(*[None for _ in COMPONENTS])


def MakeComponentTuple(*components):
    kwargs = {}
    for component in components:
        name = type(component).__name__
        kwargs[name] = component

    return __EmptyComponentTuple._replace(**kwargs)

# Special component that the entity system uses to store identifying
# information. This is kept in a component for ease of access, serialization and
# consistency reasons.
Entity = collections.namedtuple(
    "Entity",
    ["identity", "collectors"],
)


Named = collections.namedtuple(
    "Named",
    ["name", "kind"],
)


MemoryObject = collections.namedtuple(
    "MemoryObject",
    ["base_object", "type"],
)


NetworkInterface = collections.namedtuple(
    "NetworkInterface",
    ["addresses"],
)


Process = collections.namedtuple(
    "Process",
    ["pid", "parent", "user", "command", "arguments"],
)


Thread = collections.namedtuple(
    "Thread",
    ["state"],
)


Connection = collections.namedtuple(
    "Connection",
    [
        "src_addr",
        "dst_addr",
        "protocols",  # Known protocols, in order (e.g. [IPv4, TCP])
        "addressing_family",  # Addressing protocol family (e.g. INET or UNIX)
        "state",  # State of the connection, if meaningful.
        "src_bind",  # Source port for inet, vnode for unix, etc.
        "dst_bind",
    ],
)


Handle = collections.namedtuple(
    "Handle",
    ["resource", "process", "fd", "flags"],
)


File = collections.namedtuple(
    "File",
    ["full_path"],
)


User = collections.namedtuple(
    "User",
    ["uid", "home_dir", "username", "real_name"],
)


Group = collections.namedtuple(
    "Group",
    ["gid", "group_name"],
)


# Timer = collections.namedtuple(
#     "Timer",
#     [],
# )
#
#
# Route = collections.namedtuple(
#     "Route",
#     ["source", "destination", "interface"],
# )
#
#
# LoginSession = collections.namedtuple(
#     "LoginSession",
#     ["user", "timestamp"],
# )
#
#
# KernelModule = collections.namedtuple(
#     "KernelModule",
#     [],
# )
#
#
# Mount = collections.namedtuple(
#     "Mount",
#     [],
# )
#
#
# AllocationZone = collections.namedtuple(
#     "AllocationZone",
#     [],
# )
#



# This module provides for a central knowledge base which plugins can use to
# collect information.

import bisect
import logging
import re

from rekall import obj
from rekall import registry

class KernelModule(object):
    def __init__(self, session):
        self.session = session
        self.name = "nt"
        self.base = self.session.GetParameter("kernel_base")


class SymbolContainer(object):
    """A container class for symbols."""


class SortedAddresses(object):
    """An object to abstract searching for known memory locations."""

    def __init__(self):
        self._data = []
        self._map = {}

    def AddMemoryLocation(self, offset, data):
        """Adds the data to the memory offset."""
        bisect.insort_right(self._data, offset)
        self._map.setdefault(offset, []).append(data)

    def GetSpan(self, offset):
        """Gets the next lowest and next highest data below the offset."""
        idx = bisect.bisect_left(self._data, offset)

        if idx == 0:
            lowest = None
        else:
            lowest = self._map.get(self._data[idx - 1])

        if idx == len(self._data):
            highest = None
        else:
            highest = self._map.get(self._data[idx])

        return lowest, highest


class SymbolAddresses(SortedAddresses):

    def __init__(self):
        super(SymbolAddresses, self).__init__()
        self.symbols = SymbolContainer()

    def AddMemoryLocation(self, offset, func):
        """Add the function object to the list."""
        super(SymbolAddresses, self).AddMemoryLocation(offset, func)

        try:
            module_name, func_name = func.obj_name.split(":")
            module_name = re.match("[_a-zA-Z0-9]+", module_name).group(0)
            func_name = re.match("[_a-zA-Z0-9]+", func_name).group(0)

            module = getattr(self.symbols, module_name, None)
            if module is None:
                module = SymbolContainer()
                setattr(self.symbols, module_name, module)

            setattr(module, func_name, offset)

        except (ValueError, AttributeError):
            pass


class ParameterHook(object):
    """A mechanism for automatically calculating a parameter.

    The session contains many parameters which are calculated through the
    various plugins, or provided by the user. These parameters essentially
    represent a growing body of knowledge about the image we are currently
    analysing.

    Some plugins require this information before they can continue. If the
    information is already known, we do not need to re-derive it, and the value
    can be cached in the session.

    A ParameterHook is a class which is called to find out the value of a
    parameter when it is not known.
    """
    __abstract = True

    __metaclass__ = registry.MetaclassRegistry

    # The name of the parameter we will be calculating. This class will
    # automatically be called when someone accessed this name, and it is not
    # already known.
    name = None

    # The number of seconds this parameter can be assumed valid - or None if the
    # parameter does not expire. NOTE that expiry is only considered in the
    # physical_address_space.metadata("live") == True.
    expiry = None

    @classmethod
    def is_active(cls, session):
        _ = session
        return True

    def __init__(self, session):
        self.session = session

    def calculate(self):
        """Derive the value of the parameter."""


class AddressResolver(object):
    """Wrapper around a profile which allows addresses to be resolved in it."""

    # The format of a symbol name. Used by get_address_by_name().
    ADDRESS_NAME_REGEX = re.compile(
        r"(?P<deref>[*])?"              # Pointer dereference.

        r"((?P<address>0x[0-9A-Fa-f]+)|" # Alternative - Either an address, or,

        r"(?P<module>[A-Za-z_0-9\.]+)"  # Module name - can include extension
                                        # (.exe, .sys)

        r"!?"                           # ! separates module name from symbol
                                        # name.

        r"(?P<symbol>[^ +-]+)?"         # Symbol name.
        r")"                            # End alternative.

        r"(?P<op> *[+-] *)?"            # Possible arithmetic operator.
        r"(?P<offset>[0-9a-fA-Fx]+)?")  # Possible hex offset.

    def __init__(self, session):
        self.session = session
        self.vad = None
        self.profiles = {}
        self.context_cache = {}
        self.context = None
        self.Reset()

    def NormalizeModuleName(self, module):
        try:
            module_name = module.name
        except AttributeError:
            module_name = module

        module_name = unicode(module_name)
        module_name = re.split(r"[/\\]", module_name)[-1]
        result = module_name.split(".")[0]
        if result == "ntoskrnl":
            result = "nt"

        return result.lower()

    def _ParseAddress(self, name):
        m = self.ADDRESS_NAME_REGEX.match(name)
        if m:
            capture = m.groupdict()
            if not capture.get("address"):
                module = capture.get("module")
                if not module:
                    raise TypeError("Module name not specified.")

                capture["module"] = self.NormalizeModuleName(module)

            if capture["op"] and not (capture["symbol"] or
                                      capture["address"] or
                                      capture["module"]):
                raise TypeError("Operator %s must have an operand." %
                                capture["op"])

            if capture["op"] and not (capture["symbol"] or capture["address"]):
                raise TypeError(
                    "Operator %s must operate on a symbol or address." %
                    capture["op"])

            return capture

        raise TypeError("Unable to parse %r as a symbol name" % name)

    def _FindContainingModule(self, address):
        if self.modules:
            return self.modules.find_module(address)

    def _FindProcessVad(self, address):
        task = self.session.GetParameter("process_context")
        if task and self.vad:
            return self.vad.find_file_in_task(address, task)

    def FindContainingModule(self, address):
        """Finds the name of the module containing the specified address.

        Returns:
          A tuple of start address, end address, name
          """
        self._EnsureInitialized()

        # The address may be in kernel space or user space.
        containing_module = self._FindContainingModule(address)

        if containing_module:
            name = self.NormalizeModuleName(containing_module.name)
            return containing_module.base, containing_module.size, name

        # Maybe the address is in userspace.
        containing_VAD = self._FindProcessVad(address)

        # We find the vad and it
        if containing_VAD:
            start, end, name = containing_VAD
            return start, end, self.NormalizeModuleName(name)

        # If we dont know anything about the address just return Nones.
        return None, None, None

    def GetState(self):
        return dict(modules=self.modules,
                    modules_by_name=self.modules_by_name,
                    profiles=self.profiles)

    def SetState(self, state):
        self.modules = state["modules"]
        self.modules_by_name = state["modules_by_name"]
        self.profiles = state["profiles"]

    def SwitchContext(self, process):
        """Switch the address resolver to the process context."""
        # Store the current state in the context cache.
        self.context_cache[self.context] = self.GetState()
        self.context = process

        try:
            self.SetState(self.context_cache[process])
        except KeyError:
            self.Reset()

    def Reset(self):
        """Flush all caches and reset the resolver."""
        self.modules = None
        self.modules_by_name = {}
        self.profiles = {}

    def get_constant_object(self, name, target, **kwargs):
        """Instantiate the named constant with these args."""
        self._EnsureInitialized()

        profile = self.LoadProfileForName(name)
        if profile:
            components = self._ParseAddress(name)
            if not components["symbol"]:
                raise ValueError("No symbol name specified.")

            return profile.get_constant_object(
                components["symbol"],
                target=target, **kwargs)

        return obj.NoneObject("Profile for name %s unknown.", name, log=True)

    def _EnsureInitialized(self):
        if self.modules is None:
            try:
                self.modules = self.session.plugins.modules()
                for module in self.modules.lsmod():
                    module_name = self.NormalizeModuleName(module)
                    self.modules_by_name[module_name] = module

                    # Update the image base of our profiles.
                    if module_name in self.profiles:
                        self.profiles[module_name].image_base = module.base

                self.profiles["nt"] = self.session.profile
                self.modules_by_name["nt"] = KernelModule(self.session)

            except AttributeError:
                self.modules = None

        if self.vad is None and hasattr(self.session.plugins, "vad"):
            # Hold on to the vad plugin for resolving process address
            # spaces. The vad plugin maintains its own per-process cache so we
            # do not need to reset it here.
            self.vad = self.session.plugins.vad()

    def _LoadProfile(self, module_name, profile):
        self._EnsureInitialized()
        try:
            module_name = self.NormalizeModuleName(module_name)
            # Try to get the profile directly from the local cache.
            if module_name in self.profiles:
                return self.profiles[module_name]

            module = self.modules_by_name[module_name]

            module_profile = self.session.LoadProfile(profile)
            module_profile.image_base = module.base

            # Merge in the kernel profile into this profile.
            module_profile.merge(self.session.profile)

            self.profiles[module_name] = module_profile

            return module_profile
        except ValueError:
            # Cache the fact that we did not find this profile.
            self.profiles[module_name] = None
            logging.debug("Unable to resolve symbols in module %s",
                          module_name)

    def LoadProfileForDll(self, module_base, module_name):
        self._EnsureInitialized()

        if module_name in self.profiles:
            return self.profiles[module_name]

        # Create a dummy profile.
        result = obj.Profile.classes["BasicPEProfile"](
            name=module_name,
            session=self.session)

        result.image_base = module_base

        peinfo = self.session.plugins.peinfo(
            image_base=module_base, address_space=self.session.GetParameter(
                "default_address_space"))

        constants = {}
        for _, func, name, _ in peinfo.pe_helper.ExportDirectory():
            self.session.report_progress("Merging export table: %s", name)
            func_offset = func.v()
            if not result.get_constant_by_address(func_offset):
                constants[str(name or "")] = func_offset - module_base

        result.add_constants(constants_are_addresses=True, **constants)

        self.profiles[module_name] = result

        return result

    def LoadProfileForModule(self, module):
        self._EnsureInitialized()
        result = None
        module_base = module.base

        module_name = self.NormalizeModuleName(module)
        if module_name in self.profiles:
            return self.profiles[module_name]

        guid = module.RSDS.GUID_AGE
        if guid:
            result = self._LoadProfile(
                module_name, "%s/GUID/%s" % (module_name, guid))

        if not result:
            # Create a dummy profile.
            result = obj.Profile.classes["BasicPEProfile"](
                name="Dummy Profile %s" % module_name,
                session=self.session)
            result.image_base = module_base

        peinfo = self.session.plugins.peinfo(image_base=module_base,
                                             address_space=module.obj_vm)

        constants = {}
        for _, func, name, _ in peinfo.pe_helper.ExportDirectory():
            self.session.report_progress("Merging export table: %s", name)
            func_offset = func.v()
            if not result.get_constant_by_address(func_offset):
                constants[str(name or "")] = func_offset - module_base

        result.add_constants(constants_are_addresses=True, **constants)

        self.profiles[module_name] = result

        return result

    def LoadProfileForName(self, name):
        """Returns the profile responsible for the symbol name."""
        if not isinstance(name, basestring):
            raise TypeError("Name should be a string.")

        self._EnsureInitialized()

        components = self._ParseAddress(name)
        module_name = components["module"]

        # See if the user has specified the profile in the session cache.
        profile = self.session.GetParameter("%s_profile" % module_name)
        if profile:
            return self._LoadProfile(module_name, profile)

        # Try to detect the profile from the module object.
        module = self.modules_by_name.get(module_name)
        if module:
            return self.LoadProfileForModule(module)

    def get_address_by_name(self, name):
        self._EnsureInitialized()

        try:
            return int(name)
        except ValueError:
            pass

        if not isinstance(name, basestring):
            raise TypeError("Name should be a string.")

        components = self._ParseAddress(name)
        address = components["address"]
        if address:
            address = int(address, 0)

        # User did not specify an address
        if address is None:
            module = self.modules_by_name.get(components["module"])
            if module is None:
                return obj.NoneObject("No module %s" % name, log=True)

            # Found the module we use its base address
            address = module.base

        # Search for a symbol in the module.
        if components["symbol"]:
            # Get the profile for this module.
            module_profile = self.LoadProfileForModule(module)
            if module_profile:
                address = module_profile.get_constant(
                    components["symbol"], True)
            else:
                return obj.NoneObject(
                    "No profile found for module", log=True)

        # Support basic offset operations (+/-).
        if components["op"]:
            # Parse the offset as hex or decimal.
            offset = int(components["offset"], 0)
            op = components["op"].strip()
            if op == "+":
                address += offset
            elif op == "-":
                address -= offset
            else:
                raise TypeError("Operator '%s' not supported" % op)

        # If the symbol was a dereference, we need to read the address from
        # this offset.
        if components.get("deref"):
            module_profile = None
            containing_module = self._FindContainingModule(address)
            if containing_module:
                module_profile = self.LoadProfileForModule(containing_module)

            if not module_profile:
                module_profile = self.session.profile

            address = module_profile.Pointer(address).v()

        return address

    def _format_address_from_profile(self, profile, address):
        nearest_offset, name = profile.get_nearest_constant_by_address(
            address)

        if name:
            difference = address - nearest_offset
            if difference == 0:
                return "%s!%s" % (profile.name, name)
            else:
                return "%s!%s+%#x" % (profile.name, name, difference)
        else:
            return "%s!+%#x" % (profile.name, address - profile.image_base)

    def format_address(self, address, max_distance=0x1000):
        address = obj.Pointer.integer_to_address(address)

        # Try to locate the symbol below it.
        offset, name = self.get_nearest_constant_by_address(address)
        difference = address - offset

        if name:
            if difference == 0:
                return name

            # Ensure address falls within the current module.
            containing_module = self._FindContainingModule(address)
            if (containing_module and address < containing_module.end and
                0 < difference < max_distance):
                return "%s + %#x" % (
                    name, address - offset)

            else:
                hit = self._FindProcessVad(address)
                if hit:
                    start, end, name = hit
                    if start < address < end:
                        profile = self.LoadProfileForDll(start, name)
                        return self._format_address_from_profile(
                            profile, address)

        return ""

    def get_constant_by_address(self, address):
        self._EnsureInitialized()

        address = obj.Pointer.integer_to_address(address)
        containing_module = self._FindContainingModule(address)
        if containing_module:
            module_name = self.NormalizeModuleName(containing_module)
            module_profile = self.LoadProfileForName(module_name)

            if module_profile:
                constant = module_profile.get_constant_by_address(address)
                if constant:
                    return "%s!%s" % (module_name, constant)
            else:
                return module_name

        # Check the process context for process addresses.
        hit = self._FindProcessVad(address)
        if hit:
            start, _, name = hit
            return "%s + %#x" % (name, address-start)

    def get_nearest_constant_by_address(self, address):
        self._EnsureInitialized()

        address = obj.Pointer.integer_to_address(address)
        nearest_offset = 0
        full_name = module_name = symbol_name = ""
        profile = None

        # Find the containing module and see if we have a profile for it.
        containing_module = self._FindContainingModule(address)
        if containing_module:
            nearest_offset = containing_module.base
            full_name = module_name = self.NormalizeModuleName(
                containing_module)

            # Try to load the module profile.
            profile = self.LoadProfileForName(module_name)
            if profile:
                offset, name = profile.get_nearest_constant_by_address(
                    address)

                # The profile's constant is closer than the module.
                if address - offset < address - nearest_offset:
                    nearest_offset = offset
                    symbol_name = name

            if symbol_name:
                full_name = "%s!%s" % (module_name, symbol_name)
        else:
            vad_desc = self._FindProcessVad(address)
            if vad_desc:
                start, _, full_name = vad_desc
                module_name = self.NormalizeModuleName(full_name)
                nearest_offset = start
                profile = self.LoadProfileForDll(start, full_name)

                if profile:
                    offset, name = profile.get_nearest_constant_by_address(
                        address)

                    # The profile's constant is closer than the module.
                    if address - offset < address - nearest_offset:
                        nearest_offset = offset
                        symbol_name = name

                if symbol_name:
                    full_name = "%s!%s" % (module_name, symbol_name)

        return nearest_offset, full_name

    def search_symbol(self, pattern):
        # Currently we only allow searching in the same module.
        self._EnsureInitialized()
        result = []

        components = self._ParseAddress(pattern)
        module_name = components["module"]
        profile = self.LoadProfileForName(module_name)

        # Match all symbols.
        symbol_regex = re.compile(components["symbol"].replace("*", ".*"))
        for constant in profile.constants:
            if symbol_regex.match(constant):
                result.append("%s!%s" % (module_name, constant))

        return result



# Rekall Memory Forensics
# Copyright (C) 2012
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@gmail.com> based on code by
# {npetroni,awalters}@4tphi.net (Nick Petroni and AAron Walters)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""An Address Space for processing crash dump files."""
import logging

from rekall import addrspace
from rekall.plugins.overlays.windows import crashdump


class WindowsCrashDumpSpace32(addrspace.RunBasedAddressSpace):
    """ This Address Space supports windows Crash Dump format """
    order = 30

    PAGE_SIZE = 0x1000

    # Participate in Address Space voting.
    __image = True

    def __init__(self, **kwargs):
        super(WindowsCrashDumpSpace32, self).__init__(**kwargs)
        self.offset = 0
        self.fname = ''

        # Check the file for sanity.
        self.check_file()

        file_offset = self.header.size()

        for run in self.header.PhysicalMemoryBlockBuffer.Run:
            self.runs.insert((int(run.BasePage) * self.PAGE_SIZE,
                              file_offset,
                              int(run.PageCount) * self.PAGE_SIZE))

            file_offset += run.PageCount * self.PAGE_SIZE

        self.session.SetParameter(
            "dtb", int(self.header.DirectoryTableBase))

    def check_file(self):
        """Checks the base file handle for sanity."""

        self.as_assert(self.base,
                       "Must stack on another address space")

        ## Must start with the magic PAGEDUMP
        self.as_assert((self.base.read(0, 8) == 'PAGEDUMP'),
                       "Header signature invalid")

        self.profile = crashdump.CrashDump32Profile(
            session=self.session)

        self.header = self.profile.Object(
            "_DMP_HEADER", offset=self.offset, vm=self.base)

        if self.header.DumpType != "Full Dump":
            raise IOError("This is not a full memory crash dump. "
                          "Kernel crash dumps are not supported.")

    def write(self, vaddr, buf):
        # Support writes straddling page runs.
        written = 0
        while len(buf):
            file_offset, available_length = self._get_available_buffer(
                vaddr, len(buf))
            if file_offset is None:
                raise IOError("Unable to write unmapped runs yet.")

            written += self.base.write(
                file_offset, buf[:available_length])

            buf = buf[available_length:]

        return written


class WindowsCrashDumpSpace64(WindowsCrashDumpSpace32):
    """This AS supports windows Crash Dump format."""
    order = 30

    # Participate in Address Space voting.
    __image = True

    def check_file(self):
        """Check specifically for 64 bit crash dumps."""

        ## Must start with the magic PAGEDU64
        self.as_assert((self.base.read(0, 8) == 'PAGEDU64'),
                       "Header signature invalid")

        self.profile = crashdump.CrashDump64Profile(
            session=self.session)

        self.as_assert(self.profile.has_type("_DMP_HEADER64"),
                       "_DMP_HEADER64 not available in profile")
        self.header = self.profile.Object("_DMP_HEADER64",
                                          offset=self.offset, vm=self.base)

        # The following error is fatal - abort the voting mechanism.

        # Unfortunately trunk Volatility does not set this field correctly, so
        # we do not make it a fatal error. It can lead to problems if we try to
        # parse other crash dump formats, (Especially Win8 ones - see below) so
        # we might consider making this a fatal error in future.
        if self.header.DumpType != "Full Dump":
            logging.warning("This is not a full memory crash dump. "
                            "Kernel crash dumps are not supported.")

        # Catch this error early or we will hog all memory trying to parse a
        # huge number of Runs. On Windows 8 we have observed the DumpType to be
        # == 5 and these fields are padded with "PAGE" (i.e. 0x45474150).
        if self.header.PhysicalMemoryBlockBuffer.NumberOfRuns > 100:
            raise RuntimeError(
                "This crashdump file format is not supported. Rekall does not "
                "currently support crashdumps using the Win8 format.")



# Import all the renderers.
# pylint: disable=unused-import

from rekall import config
from rekall.ui import renderer
from rekall.ui import json_renderer
from rekall.ui import text

class JITIterator(object):
    def __contains__(self, item):
        return item in list(self)

    def __iter__(self):
        return (
            x.name for x in renderer.BaseRenderer.classes.values() if x.name)


config.DeclareOption(
    "-r", "--renderer", default="text", group="Interface",
    choices=JITIterator(), help="The renderer to use. Default (text)")




#!/usr/bin/python

# Rekall Memory Forensics
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""Support IPython 1.0."""

# pylint: disable=protected-access

__author__ = "Michael Cohen <scudette@gmail.com>"
import logging
import re
import readline

from rekall import config
from rekall import constants

try:
    from IPython.terminal import embed
except ImportError:
    try:
        from IPython.frontend.terminal import embed
    except ImportError:
        embed = None

from IPython.config.loader import Config


config.DeclareOption("--ipython_engine",
                     help="IPython engine, e.g. notebook.")


def RekallCompleter(self, text):
    """Sophisticated command line completer for Rekall."""
    try:
        command_parts = self.line_buffer.split(" ")
        command = command_parts[0]

        if command.startswith("plugins."):
            command = command[len("plugins."):]

        global_matches = set(self.global_matches(command))

        # Complete strings which look like symbol names.
        m = re.match("\"([^!]+![^\"]*)$", command_parts[-1])
        if m:
            session = self.namespace.get("session")

            # If this is the only match, close the quotes to save typing.
            result = session.address_resolver.search_symbol(m.group(1) + "*")
            if len(result) == 1:
                result = [result[0] + "\""]

            result = [x.split("!", 1)[1] for x in result]

            return result

        # Only complete if there is exactly one object which matches and a space
        # was typed after it. e.g.: pslist <cursor>
        if (global_matches and len(global_matches) == 1 and
            len(command_parts) > 1):

            # Get the object and ask it about the list of default args.
            obj = self.namespace.get(global_matches.pop())
            try:
                matches = ["%s=" % x for x in obj.get_default_arguments()]
                return [x for x in matches if x.startswith(text)]
            except Exception:
                pass

        return []

    # Wide exception is necessary here because otherwise the completer will
    # swallow all errors.
    except Exception as e:
        logging.debug(e)


def Shell(user_session):
    # This should bring back the old autocall behaviour. e.g.:
    # In [1]: pslist
    cfg = Config()
    cfg.InteractiveShellEmbed.autocall = 2

    cfg.PromptManager.in_template = (
        r'{color.LightCyan}'
        r'{session.state.base_filename}'
        r'{color.LightBlue}{color.Green} \T> ')

    cfg.PromptManager.in2_template = (
        r'{color.Green}|{color.LightGreen}\D{color.Green}> ')

    cfg.PromptManager.out_template = r'Out<\#> '
    cfg.InteractiveShell.separate_in = ''
    cfg.InteractiveShell.separate_out = ''
    cfg.InteractiveShell.separate_out2 = ''

    shell = embed.InteractiveShellEmbed(
        config=cfg, user_ns=user_session._locals)

    shell.Completer.merge_completions = False
    shell.banner = constants.BANNER
    shell.exit_msg = constants.GetQuote()
    shell.set_custom_completer(RekallCompleter, 0)

    # Do we need to pre-run something?
    if user_session.run != None:
        execfile(user_session.run, user_session._locals)

    # Workaround for completer bug.
    import IPython.core.completerlib
    IPython.core.completerlib.get_ipython = lambda: shell

    # Set known delimeters for the completer. This varies by OS so we need to
    # set it to ensure consistency.
    readline.set_completer_delims(' \t\n`!@#$^&*()=+[{]}\\|;:\'",<>?')

    shell(local_ns=user_session._locals)

    return True



def NotebookSupport(_):

    # The following only reveals hidden imports to pyinstaller.
    if False:
        # pylint: disable=unused-variable
        import IPython.html.auth.login
        import IPython.html.auth.logout
        import IPython.html.base.handlers
        import IPython.html.nbconvert.handlers
        import IPython.html.notebook.handlers
        import IPython.html.notebookapp
        import IPython.html.services.clusters.handlers
        import IPython.html.services.kernels.handlers
        import IPython.html.services.nbconvert.handlers
        import IPython.html.services.notebooks.handlers
        import IPython.html.services.sessions.handlers
        import IPython.html.tree.handlers
        import IPython.kernel.ioloop
        import IPython.kernel.zmq.kernelapp

        import rekall.interactive

        import zmq.backend.cython
        import zmq.eventloop.ioloop

    argv = ["notebook", "-c",
            "from rekall import interactive; "
            "interactive.ImportEnvironment();", "--autocall", "2",
            "--notebook-dir",
            config.GetConfigFile().get("notebook_dir",
                                       config.GetHomeDir())
            ]

    import IPython

    IPython.start_ipython(argv=argv)
    return True



# Rekall Memory Forensics
# Copyright (C) 2012 Michael Cohen
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""This module implements the rekall session.

The session stores information about the a specific user interactive
session. Sessions can be saved and loaded between runs and provide a convenient
way for people to save their own results.
"""

__author__ = "Michael Cohen <scudette@gmail.com>"

import json
import logging
import os
import pdb
import sys
import time
import traceback

from rekall import addrspace
from rekall import args
from rekall import config
from rekall import constants
from rekall import entity
from rekall import io_manager
from rekall import kb
from rekall import obj
from rekall import plugin
from rekall import registry
from rekall import utils

from rekall.ui import renderer
from rekall.ui import json_renderer


# Top level args.
config.DeclareOption("-p", "--profile", group="Autodetection Overrides",
                     help="Name of the profile to load. This is the "
                     "filename of the profile found in the profiles "
                     "directory. Profiles are searched in the profile "
                     "path order.")

config.DeclareOption(
    "--profile_path", default=[], action="append",
    help="Path to search for profiles. This can take "
    "any form supported by the IO Manager (e.g. zip files, "
    "directories, URLs etc)")


config.DeclareOption("-f", "--filename",
                     help="The raw image to load.")

config.DeclareOption(
    "--buffer_size", default=20*1024*1024,
    action=config.IntParser,
    help="The maximum size of buffers we are allowed to read. "
    "This is used to control Rekall memory usage.")

config.DeclareOption(
    "--output", default=None,
    help="If specified we write output to this file.")


class Container(object):
    """Just a container."""

    def __str__(self):
        return str(self.__dict__.keys())


class Cache(utils.AttributeDict):
    def Get(self, item, default=None):
        if default is None:
            default = obj.NoneObject("%s not found in cache.", item)

        return super(Cache, self).Get(item) or default

    def __getattr__(self, attr):
        # Do not allow private attributes to be set.
        if attr.startswith("_"):
            raise AttributeError(attr)

        res = self.get(attr)
        if res is None:
            return obj.NoneObject("%s not set in cache", attr)

        return res

    def __str__(self):
        result = []
        for k, v in self.iteritems():
            if isinstance(v, obj.BaseObject):
                v = repr(v)

            value = "\n  ".join(str(v).splitlines())
            if len(value) > 1000:
                value = "%s ..." % value[:1000]

            result.append("  %s = %s" % (k, value))

        return "{\n" + "\n".join(sorted(result)) + "\n}"


class Configuration(Cache):
    # The session which owns this configuration object.
    session = None

    # This holds a write lock on the configuration object.
    _lock = False
    _pending_hooks = None

    def __init__(self, session=None, **kwargs):
        super(Configuration, self).__init__(**kwargs)
        self.session = session
        self.update(**kwargs)
        self._pending_hooks = []

        # Can not update the configuration object any more.
        self._lock = 1


    def __repr__(self):
        return "<Configuration Object>"

    def _set_filename(self):
        if self.get('filename') != self.filename:
            self['filename'] = self.filename
            if self.session:
                self.session.Reset()
                self.session.UpdateFromConfigObject()

        self['base_filename'] = os.path.basename(self.filename)

    def _set_profile_path(self):
        # Flush the profile cache if we change the profile path.
        self['profile_path'] = self.profile_path
        self.session.profile_cache = {}
        self.session.UpdateFromConfigObject()

    def _set_profile(self):
        profile = self.Get("profile")
        if isinstance(profile, basestring):
            loaded_profile = self.session.LoadProfile(profile)
            if loaded_profile:
                with self:
                    self.Set("profile", loaded_profile)

        # The profile has changed - update the active plugin list.
        self.session.UpdateRunners()

    def _set_logging(self):
        level = self.logging
        if isinstance(level, basestring):
            level = getattr(logging, level.upper(), logging.INFO)

        if level == None:
            return

        logging.info("Logging level set to %s", level)
        logging.getLogger().setLevel(int(level))

    def Set(self, attr, value):
        if self._lock > 0:
            raise ValueError(
                "Can only update configuration using the context manager.")

        hook = getattr(self, "_set_%s" % attr, None)
        if hook:
            self._pending_hooks.append(hook)

        super(Configuration, self).Set(attr, value)

    def __enter__(self):
        # Allow us to update the context manager.
        self._pending_hooks = []
        self._lock -= 1

        return self

    def __exit__(self, exc_type, exc_value, trace):
        self._lock += 1

        # Run all the hooks _after_ all the parameters have been set.
        if self._lock == 1:
            for hook in reversed(self._pending_hooks):
                hook()


class ProgressDispatcher(object):
    """An object to manage progress calls.

    Since Rekall must be usable as a library it can not block for too
    long. Rekall makes continuous reports of its progress to the
    ProgressDispatcher, which then further dispatches them to other
    callbacks. This allows users of the Rekall library to be aware of how
    analysis is progressing. (e.g. to report it in a GUI).

    """

    def __init__(self):
        self.heap = []
        self.callbacks = {}

    def Register(self, key, callback):
        self.callbacks[key] = callback

    def UnRegister(self, key):
        self.callbacks.pop(key, 0)

    def Broadcast(self, message, *args, **kwargs):
        for handler in self.callbacks.values():
            handler(message, *args, **kwargs)


class Session(object):
    """Base session.

    This session contains the bare minimum to use rekall.
    """

    __metaclass__ = registry.MetaclassRegistry

    def __init__(self, **kwargs):
        self._parameter_hooks = {}

        # Store user configurable attributes here. These will be read/written to
        # the configuration file.
        kwargs.setdefault("cache", Cache())
        self.state = Configuration(self, **kwargs)

        self.progress = ProgressDispatcher()
        self.profile = obj.NoneObject("Set this to a valid profile "
                                      "(e.g. type profiles. and tab).")

        # Cache the profiles we get from LoadProfile() below.
        # TODO: This should probably be also done on disk somewhere to avoid
        # having to hit the profile repository all the time.
        self.profile_cache = {}

        self.entities = entity.EntityManager(session=self)

        self.inventories = {}
        self.UpdateFromConfigObject()
        self._configuration_parameters = [x[2] for x in config.OPTIONS]

    def __enter__(self):
        # Allow us to update the context manager.
        self.state.__enter__()
        return self

    def __exit__(self, exc_type, exc_value, trace):
        self.state.__exit__(exc_type, exc_value, trace)

    def Reset(self):
        self.profile_cache = {}
        self.physical_address_space = None
        self.kernel_address_space = None
        self.state.cache.clear()

    def UpdateFromConfigObject(self):
        """This method is called whenever the config object was updated.

        We are expected to re-check the config and re-initialize this session.
        """
        self.UpdateRunners()

        filename = self.state.filename
        if filename:
            # This may fire off the profile auto-detection code if a profile was
            # not provided by the user.
            profile_parameter = self.GetParameter("profile")
            if profile_parameter:
                self.profile = self.LoadProfile(profile_parameter)
                if self.profile == None:
                    raise ValueError(self.profile.reason)

                # The profile has just changed, we need to update the runners.
                self.UpdateRunners()

        # Set the renderer.
        self.renderer = renderer.BaseRenderer.classes.get(
            self.GetParameter("renderer"), "TextRenderer")

        # Make a new address resolver.
        self.address_resolver = kb.AddressResolver(self)

    def UpdateRunners(self):
        """Updates the plugins container with active plugins.

        Active plugins may change based on the profile/filename etc.
        """
        self.plugins = Container()
        for cls in plugin.Command.GetActiveClasses(self):
            name = cls.name
            if name:
                setattr(self.plugins, name, obj.Curry(cls, session=self))

        # Install parameter hooks.
        self._parameter_hooks = {}
        for cls in kb.ParameterHook.classes.values():
            if cls.is_active(self) and cls.name:
                self._parameter_hooks[cls.name] = cls(session=self)

    def __getattr__(self, attr):
        """This will only get called if the attribute does not exist."""
        return obj.NoneObject("Attribute not set")

    def GetParameter(self, item, default=obj.NoneObject()):
        """Retrieves a stored parameter.

        Parameters are managed by the Rekall session in two layers. The state
        containers contains those parameters which are deliberately set by the
        user.

        Some parameters are calculated by plugins and are used in order to speed
        up further calculations. These are cached in the state as well.

        It is important to never override a user selection by the cached
        results. Since the user must be allowed to override all parameters - for
        example through the GUI or the command line. Therefore when resolving a
        parameter, we first check in the state, and only if the parameter does
        not exist, we check the cache.
        """
        # The state holds user configuration from ~/.rekallrc.
        result = self.state.Get(item)
        if result == None:
            # self.state.cache holds cached parameters. If the item is already
            # in the cache just return it, otherwise run the parameter hook.
            if item in self.state.cache:
                result = self.state.cache[item]
            else:
                result = self._RunParameterHook(item)

        if result == None:
            result = default

        return result

    def SetParameter(self, item, value):
        # Configuration parameters go in the state object, everything else goes
        # in the cache.
        if item in self._configuration_parameters:
            self.state.Set(item, value)

        else:
            self.state.cache.Set(item, value)

    def _RunParameterHook(self, name):
        hook = self._parameter_hooks.get(name)
        if hook:
            result = hook.calculate()
            with self:
                self.SetParameter(name, result)

            return result

    def RunPlugin(self, plugin_obj, *pos_args, **kwargs):
        """Launch a plugin and its render() method automatically.

        We use the pager specified in session.GetParameter("pager").

        Args:
          plugin_obj: A string naming the plugin, or the plugin instance itself.
          *pos_args: Args passed to the plugin if it is not an instance.
          **kwargs: kwargs passed to the plugin if it is not an instance.
        """
        flags = kwargs.pop("flags", None)
        output = kwargs.pop("output", None)
        renderer = kwargs.pop("renderer", None)

        if isinstance(plugin_obj, basestring):
            plugin_name = plugin_obj
        elif isinstance(plugin_obj, plugin.Command):
            plugin_name = plugin_obj.name
        else:
            raise TypeError(
                "First parameter should be a plugin name or instance.")

        # When passed as a string this specifies a plugin name.
        if isinstance(plugin_obj, basestring):
            plugin_cls = getattr(self.plugins, plugin_obj, None)
            if plugin_cls is None:
                logging.error(
                    "Plugin %s is not active. Is it supported with "
                    "this profile?", plugin_name)
                return

            # If the args came from the command line parse them now:
            if flags:
                kwargs = args.MockArgParser().build_args_dict(
                    plugin_cls, flags)

            # Instantiate the plugin object.
            plugin_obj = plugin_cls(*pos_args, **kwargs)

        # Do we need to redirect output?
        if output is not None:
            with self:
                # Do not lose the global output redirection.
                old_output = self.GetParameter("output") or None
                self.SetParameter("output", output)
                try:
                    return self._RunPlugin(plugin_obj, **kwargs)
                finally:
                    self.SetParameter("output", old_output)

        else:
            return self._RunPlugin(plugin_obj, renderer=renderer, **kwargs)

    def _RunPlugin(self, plugin_obj, **kwargs):
        ui_renderer = kwargs.pop("renderer", None)
        if ui_renderer is None:
            ui_renderer = self.GetRenderer()

        # Start the renderer before instantiating the plugin to allow
        # rendering of reported progress in the constructor.
        with ui_renderer.start(plugin_name=plugin_obj.name, kwargs=kwargs):
            try:
                plugin_obj.render(ui_renderer)
                return plugin_obj

            except plugin.InvalidArgs as e:
                logging.error("Invalid Args (Try 'info plugins.%s'): %s",
                              plugin_obj.name, e)

            except plugin.PluginError as e:
                ui_renderer.report_error(str(e))

            except KeyboardInterrupt:
                ui_renderer.report_error("Aborted")
                self.report_progress("Aborted!\r\n", force=True)

            except Exception, e:
                # If anything goes wrong, we break into a debugger here.
                ui_renderer.report_error(traceback.format_exc())

                if self.GetParameter("debug"):
                    pdb.post_mortem(sys.exc_info()[2])

                raise

            return plugin_obj

    def LoadProfile(self, filename, use_cache=True):
        """Try to load a profile directly from a filename.

        Args:
          filename: A string which will be used to get an io_manager
            container. If it contains a path sepearator we open the file
            directly, otherwise we search in the profile_path specification.

        Returns:
          a Profile() instance or a NoneObject()
        """
        if not filename:
            return

        if isinstance(filename, obj.Profile):
            return filename

        # We only want to deal with unix paths.
        filename = filename.replace("\\", "/")

        # Only strip the extension if it is one of the recognized
        # extensions. Otherwise ignore it - this allows the profile name to have
        # . characters in it (e.g. Linux-3.1.13).
        canonical_name, extension = os.path.splitext(filename)
        if extension not in [".gz", ".json"]:
            canonical_name = filename

        try:
            if use_cache:
                cached_profile = self.profile_cache[canonical_name]
                if cached_profile:
                    return cached_profile

                else:
                    return obj.NoneObject(
                        "Unable to load profile %s from any repository." %
                        filename)

        except KeyError:
            pass

        # If the filename is a path we try to open it directly:
        if os.access(filename, os.R_OK):
            container = io_manager.Factory(os.path.dirname(filename))
            result = obj.Profile.LoadProfileFromData(
                container.GetData(os.path.basename(filename)),
                self, name=canonical_name)

        # Traverse the profile path until one works.
        else:
            result = None

            # The profile path is specified in search order.
            profile_path = self.state.Get("profile_path") or []

            # Add the last supported repository as the last fallback path.
            for path in profile_path:
                path = "%s/%s" % (path, constants.PROFILE_REPOSITORY_VERSION)

                # For now, print a warning when the user has an out of date
                # config file. TODO: Remove this in a future version.
                if "profiles.rekall.googlecode.com" in path:
                    logging.warn(
                        "Rekall profiles have moved to %s, but your .rekallrc "
                        "file still points to %s. You should update your "
                        "config file.",
                        constants.PROFILE_REPOSITORIES[0],
                        path)

                try:
                    manager = io_manager.Factory(path)
                    try:
                        # The inventory allows us to fail fetching the profile
                        # quickly - without making the round trip.
                        if path not in self.inventories:
                            # Fetch the profile inventory.
                            self.inventories[path] = manager.GetData(
                                "inventory")

                        inventory = self.inventories[path]["$INVENTORY"]
                        if (filename not in inventory and
                            filename + ".gz" not in inventory):
                            continue

                    # No inventory in that repository - just try anyway.
                    except IOError:
                        pass

                    result = obj.Profile.LoadProfileFromData(
                        manager.GetData(filename), self,
                        name=canonical_name)
                    logging.info(
                        "Loaded profile %s from %s", filename, manager)

                    break

                except (IOError, KeyError) as e:
                    result = obj.NoneObject(e)
                    logging.debug("Could not find profile %s in %s",
                                  filename, path)

                    continue

        # Cache it for later. Note that this also caches failures so we do not
        # retry again.
        self.profile_cache[canonical_name] = result
        if result == None:
            return obj.NoneObject(
                "Unable to load profile %s from any repository." % filename)

        return result

    def __unicode__(self):
        return u"Session"

    def report_progress(self, message=" %(spinner)s", *args, **kwargs):
        """Called by the library to report back on the progress."""
        self.progress.Broadcast(message, *args, **kwargs)

    def GetRenderer(self):
        """Get a renderer for this session.

        We instantiate the renderer specified in self.GetParameter("renderer").
        """
        ui_renderer = self.GetParameter("renderer", "text")
        if isinstance(ui_renderer, basestring):
            ui_renderer_cls = renderer.BaseRenderer.ImplementationByName(
                ui_renderer)
            ui_renderer = ui_renderer_cls(session=self)

        return ui_renderer

    @property
    def profile(self):
        res = self.state.Get("profile")
        return res

    @profile.setter
    def profile(self, value):
        super(Configuration, self.state).Set("profile", value)


class JsonSerializableSession(Session):
    """A session which can serialize its state into a Json file."""

    def SaveToFile(self, filename):
        with open(filename, "wb") as fd:
            logging.info("Saving session to %s", filename)
            json.dump(self.Serialize(), fd)

    def LoadFromFile(self, filename):
        try:
            lexicon, data = json.load(open(filename, "rb"))
            logging.info("Loaded session from %s", filename)

            self.Unserialize(lexicon, data)

        # decoding the session might fail un-expectantly - just discard the
        # session in that case.
        except Exception:
            # If anything goes wrong, we break into a debugger here.
            logging.error(traceback.format_exc())

            if self.GetParameter("debug"):
                pdb.post_mortem(sys.exc_info()[2])

    def Unserialize(self, lexicon, data):
        json_renderer_obj = json_renderer.JsonRenderer(session=self)
        decoder = json_renderer.JsonDecoder(self, json_renderer_obj)
        decoder.SetLexicon(lexicon)
        self.state = Configuration(**decoder.Decode(data))
        self.UpdateFromConfigObject()

    def Serialize(self):
        encoder = json_renderer.JsonEncoder(
            session=self, renderer="JsonRenderer")

        data = encoder.Encode(self.state)
        return encoder.GetLexicon(), data


class InteractiveSession(JsonSerializableSession):
    """The session allows for storing of arbitrary values and configuration.

    This session contains a lot of convenience features which are useful for
    interactive use.
    """

    def __init__(self, env=None, **kwargs):
        self._locals = env or {}

        # These are the command plugins which we exported to the local
        # namespace.
        self._start_time = time.time()

        # These keep track of the last run plugin.
        self._last_plugin = None

        # Fill the session with helpful defaults.
        self.pager = obj.NoneObject("Set this to your favourite pager.")

        self.help_profile = None

        super(InteractiveSession, self).__init__(**kwargs)

    def UpdateRunners(self):
        super(InteractiveSession, self).UpdateRunners()

        if not self.help_profile:
            self.help_profile = self.LoadProfile("help_doc")

        self._locals['plugins'] = Container()
        for cls in plugin.Command.GetActiveClasses(self):
            default_args, doc = "", ""
            if self.help_profile:
                default_args = self.help_profile.ParametersForPlugin(
                    cls.__name__)
                doc = self.help_profile.DocsForPlugin(cls.__name__)

            name = cls.name
            if name:
                # Create a runner for this plugin and set its documentation.
                runner = obj.Curry(
                    self.RunPlugin, name, default_arguments=default_args)

                runner.__doc__ = doc

                setattr(self._locals['plugins'], name, runner)
                self._locals[name] = runner

    def reset(self):
        """Reset the current session by making a new session."""
        self._prepare_local_namespace()

    def RunPlugin(self, *args, **kwargs):
        self.last = super(InteractiveSession, self).RunPlugin(*args, **kwargs)

    def _prepare_local_namespace(self):
        session = self._locals['session'] = self
        # Prepopulate the namespace with our most important modules.
        self._locals['addrspace'] = addrspace
        self._locals['obj'] = obj
        self._locals['profile'] = self.profile
        self._locals['v'] = session.v

        # Add all plugins to the local namespace and to their own container.
        self.UpdateRunners()

        # Some useful modules which should be available always.
        self._locals["sys"] = sys
        self._locals["os"] = os

    def v(self):
        """Re-execute the previous command."""
        if self.last:
            self.RunPlugin(self.last)

    def lister(self, arg):
        for x in arg:
            self.printer(x)

    def __str__(self):
        result = """Rekall Memory Forensics session Started on %s.

Config:
%s
""" % (time.ctime(self._start_time), self.state)
        return result

    def __dir__(self):
        items = self.__dict__.keys() + dir(self.__class__)

        return [x for x in items if not x.startswith("_")]

    def error(self, plugin_cls, e):
        """Swallow the error but report it."""
        logging.error("Failed running plugin %s: %s",
                      plugin_cls.name, e)



# Rekall Memory Forensics
#
# Copyright 2014 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""
Common Darwin collectors.
"""
__author__ = "Adam Sindelar <adamsh@google.com>"

from rekall import components
from rekall import identity


def ParseProcess(proc):
    process = components.Process(
        pid=proc.pid,
        command=proc.p_comm,
        parent=identity.ProcessIdentity(pid=proc.p_ppid),
        user=identity.UserIdentity(uid=proc.p_uid),
        arguments=None,
    )

    named = components.Named(
        name="PID %d" % proc.pid,
        kind="Process",
    )

    memory_obj = components.MemoryObject(
        base_object=proc,
        type="proc",
    )

    user = components.User(
        uid=proc.p_uid,
        username=None,
        home_dir=None,
        real_name=None,
    )

    user_named = components.Named(
        name=None,
        kind="User",
    )

    yield identity.ProcessIdentity(pid=proc.pid), [process, named, memory_obj]
    yield identity.UserIdentity(uid=proc.p_uid), [user, user_named]


def DarwinPgrpHashProcessCollector(profile):
    """Generates Process entities using hash of process groups.

    Adapted from legacy pslist plugin's list_using_pgrp_hash.

    XNU Reference:
      xnu-1699.26.8/bsd/sys/proc_internal.h
    """
    # Note that _pgrphash is initialized through:
    #
    # xnu-1699.26.8/bsd/kern/kern_proc.c:195
    # hashinit(int elements, int type, u_long *hashmask)
    #
    # /xnu-1699.26.8/bsd/kern/kern_subr.c: 327
    # hashinit(int elements, int type, u_long *hashmask) {
    #    ...
    # *hashmask = hashsize - 1;
    #
    # Hence the value in _pgrphash is one less than the size of the hash
    # table.

    pgr_hash_table = profile.get_constant_object(
        "_pgrphashtbl",
        target="Pointer",
        target_args=dict(
            target="Array",
            target_args=dict(
                target="pgrphashhead",
                count=profile.get_constant_object(
                    "_pgrphash", "unsigned long") + 1
            )
        )
    )

    for slot in pgr_hash_table.deref():
        for pgrp in slot.lh_first.walk_list("pg_hash.le_next"):
            for proc in pgrp.pg_members.lh_first.walk_list(
                "p_pglist.le_next"):
                for result in ParseProcess(proc):
                    yield result


def DarwinTaskProcessCollector(profile):
    """List processes using the processor tasks queue.

    Adapted from legacy pslist plugin's list_using_task.

    XNU reference:
      /osfmk/kern/processor.c (processor_set_things)
    """
    tasks = profile.get_constant_object(
        "_tasks",
        target="queue_entry",
        vm=profile.session.kernel_address_space
    )

    for task in tasks.list_of_type("task", "tasks"):
        proc = task.bsd_info.deref()
        if proc:
            for result in ParseProcess(proc):
                yield result


def DarwinAllprocProcessCollector(profile):
    """List all processes by following the _allproc list head.

    Adapted from legacy pslist plugin's list_using_allproc.

    References TBD.
    """
    allproc = profile.get_constant_object(
        "_allproc", target="proclist")
    for proc in allproc.lh_first.p_list:
        for result in ParseProcess(proc):
            yield result


def DarwinPidHashProcessCollector(profile):
    """List processes using pid hash tables.

    Adapted from legacy pslist plugin's list_using_pid_hash.

    XNU reference:
      xnu-1699.26.8/bsd/kern/kern_proc.c:834:
    """
    # Note that _pidhash is initialized through:
    #
    # xnu-1699.26.8/bsd/kern/kern_proc.c:194
    # pidhashtbl = hashinit(maxproc / 4, M_PROC, &pidhash);
    #
    # /xnu-1699.26.8/bsd/kern/kern_subr.c: 327
    # hashinit(int elements, int type, u_long *hashmask) {
    #    ...
    # *hashmask = hashsize - 1;
    #
    # Hence the value in pidhash is one less than the size of the hash
    # table.

    pid_hash_table = profile.get_constant_object(
        "_pidhashtbl",
        target="Pointer",
        target_args=dict(
            target="Array",
            target_args=dict(
                target="pidhashhead",
                count=profile.get_constant_object(
                    "_pidhash", "unsigned long") + 1
            )
        )
    )

    for plist in pid_hash_table.deref():
        for proc in plist.lh_first.walk_list("p_hash.le_next"):
            if proc:
                for result in ParseProcess(proc):
                    yield result




# Rekall Memory Forensics
#
# Copyright 2014 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""
The Rekall Memory Forensics entity layer.
"""
__author__ = "Adam Sindelar <adamsh@google.com>"

from rekall import obj

class Identity(object):
    """Uniquely identifies something like a process or a user.

    When collecting information about a system, data that relates to the same
    conceptual entity, such as a user or a network connection, can come from
    multiple sources. In such cases, this class provides a way of uniquelly
    identifying the conceptual entities (such as users, processes, connections,
    etc.) to make it easier to view information about them in one place, even
    when it was collected using different algorithms.

    Identities can be either scalar or vector. Scalar identities wrap around a
    single piece of information, typically a numeric ID or a memory offset and
    use that for comparisons and hashing. Vector identities have more than one
    way of identifying the object. For example, a user may be identified by
    either their UID or their username, with each being equally valid and
    unique (within the scope of one system).
    """
    def __init__(self):
        pass

    def __eq__(self, other):
        """Is this identity equivalent to the other identity?

        Returns:
            Scalar identities should return True or False (never None).

            Vector identities may return None when the question is
            undecidable right now, but may be decidable at a later time.

            Example - let x be a user identified by the UID 37 and y be a user
            identified by the username "Alice". X and y both posses valid
            identifiers of a user, but they are not comparable until we set
            username on x or UID on y.
        """
        pass

    def __ne__(self, other):
        """Three-valued not equal predicate."""
        eq = self.__eq__(other)

        if eq is None:
            return None  # Three-valued logic sentinel for unknown.

        return not eq

    def union(self, other):
        """Return a new identity that's a union of self and other.

        Self and other should be equivalent (self.__eq__(other) returns True).

        Returns:
            Scalar identities should return self.

            Vector identities should return a new identity instance,
            combining the known keys from self and other in a union of both.

        Note on immutability:
            Identities are immmutable. If you override this method you MUST
            return a new identity and you MUST NOT modify self or other.
        """
        _ = other
        return self

    @property
    def indices(self):
        """Set of hashable keys that can be used to index this identity.

        Returns:
            Scalar identities should implement this same as __hash__, with one
            difference: instead of the value itself, return the value wrapped
            in a tuple like so: (value,)

            Vector identities should return a set (or other iterable) of
            valid key values. Usually, this is one per non-null member of the
            identity vector/tuple.
        """
        return set([self.__unicode__()])

    def __hash__(self):
        """Hashes the identity with a caveat. Don't use, unless you know how.

        WARNING: This has an unexpected property, which is that composite
        entities may be equal to each other and yet have different hashes. This
        probably breaks whatever scheme you had in mind that used entities as
        keys. My advice to you is to just use EntityManager and not worry about
        it. The only reason __hash__ is implemented on identity is because it
        allows for some optimizations with sets in EntityLookupTable.
        """
        return hash(self.__unicode__())

    def __unicode__(self):
        pass

    def __str__(self):
        return self.__unicode__()

    def __repr__(self):
        return self.__unicode__()

    def __or__(self, other):
        return self.union(other)


class BaseObjectIdentity(Identity):
    """Implements Identity using BaseObject memory locations.

    This container will store:
      obj_offset
      obj_type
      obj_vm, actually an AttributeDict with:
        dtb (or 0 for physical address space)

    The interface above is the same as what BaseObject would provide which
    is useful when using the two interchangeably.

    This class also supports pickling and, once unpickled, can rebuild the
    original BaseObject instance using an active session.

    TODO: This class is going to be drastically simplified.
    """
    def __init__(self, base_obj, follow_pointers=True):
        super(BaseObjectIdentity, self).__init__()

        # Dereference pointers by default so we don't have the same thing
        # appear more than one time. One exception is void pointers.
        if (follow_pointers and
            isinstance(base_obj, obj.Pointer) and
            base_obj.target != "void"):
            base_obj = base_obj.dereference()

        self.obj_offset = base_obj.obj_offset
        self.obj_type = base_obj.obj_type
        self.dtb = getattr(base_obj.obj_vm, "dtb", 0)

        if isinstance(base_obj, obj.Pointer) and follow_pointers:
            _base_obj = base_obj.dereference()
            if _base_obj:
                base_obj = base_obj

    def restore_base_obj(self, session):
        """Rebuild the original BaseObject instance."""
        if self.dtb == 0:
            vm = session.physical_address_space
        elif self.dtb == session.kernel_address_space.dtb:
            vm = session.kernel_address_space
        else:
            raise AttributeError(
                "DTB does't match the kernel virtual address space."
            )

        if session == None:
            raise AttributeError(
                "Cannot restore a base object without a valid session."
            )

        return session.profile.Object(
            type_name=self.obj_type,
            offset=self.obj_offset,
            vm=vm,
        )

    def __eq__(self, other):
        if not isinstance(other, BaseObjectIdentity):
            return False

        return ((self.obj_type, self.obj_offset, self.dtb)
                == (other.obj_type, other.obj_offset, other.dtb))

    def __unicode__(self):
        return "<%s 0x%x; dtb:0x%x>" % (
            self.obj_type,
            self.obj_offset,
            self.dtb,
        )

    @property
    def indices(self):
        return set([(self.obj_type, self.obj_offset, self.dtb)])


class UserIdentity(Identity):
    def __init__(self, uid=None, username=None):
        super(UserIdentity, self).__init__()
        self.uid = uid
        self.username = username

    def __eq__(self, other):
        if not other:
            return False

        if ((self.uid == None and other.username == None) or
            (self.username == None and other.uid == None)):
            return None

        return (
            (self.uid == other.uid != None) or
            (self.username == other.username != None)
        )

    def union(self, other):
        return UserIdentity(
            uid=self.uid or other.uid,
            username=self.username or other.username,
        )

    @property
    def indices(self):
        if self.uid:
            yield "uid/%d" % self.uid

        if self.username:
            yield "username/%s" % self.username

    def __unicode__(self):
        return "User %s:%d" % (self.username, self.uid)


class ProcessIdentity(Identity):
    def __init__(self, pid=None):
        super(ProcessIdentity, self).__init__()
        self.pid = pid

    def __eq__(self, other):
        return other and self.pid == other.pid

    @property
    def indices(self):
        return ("pid/%d" % self.pid,)

    def __unicode__(self):
        return "Process %d" % self.pid





# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

"""
Networking plugins that are not OS-specific live here.
"""
__author__ = "Adam Sindelar <adamsh@google.com>"

from rekall import plugin


class EntityIFConfig(plugin.ProfileCommand):
    """List active network interfaces and their addresses."""

    __name = "eifconfig"

    def render(self, renderer):
        renderer.table_header([("Interface", "interface", "10"),
                               ("Family", "af", "10"),
                               ("Address", "address", "20")],
                              sort=("interface",))

        for interface in self.session.entities.find_by_component(
            component="NetworkInterface",
        ):
            for address in interface.components.NetworkInterface.addresses:
                renderer.table_row(
                    interface.components.Named.name,
                    address[0],
                    address[1],
                )


class EntityNetstat(plugin.ProfileCommand):
    """List per process network connections."""

    __name = "enetstat"

    def render(self, renderer):
        entities = self.session.entities.find_by_component("Connection")

        # Sort all the sockets into buckets based on whether they're network or
        # UNIX connections.
        inet_socks = []
        unix_socks = []

        for entity in entities:
            af = entity["Connection.addressing_family"]
            if af in ("AF_INET", "AF_INET6"):
                # Inet socket.
                row = [
                    entity["Connection.protocols"][1],
                    entity["Connection.src_addr"],
                    entity["Connection.src_bind"],
                    entity["Connection.dst_addr"],
                    entity["Connection.dst_bind"],
                    entity["Connection.state"],
                ]

                bucket = inet_socks
            elif af == "AF_UNIX":
                # Unix socket
                row = [
                    entity["Connection.src_addr"],
                    entity["Connection.dst_addr"],
                    entity["Connection.protocols"][1],
                    entity["Connection.src_bind"],
                ]

                bucket = unix_socks
            else:
                continue  # Don't care.

            # A single socket can be open by 1 (most common), 0 or multiple
            # handles (fds) owned by one or more processes.
            handles = list(entity.get_referencing_entities("Handle.resource"))
            for handle in handles:
                proc = handle["Handle.process"]
                bucket.append(row + proc["Process.pid", "Process.command"])

            if not handles:
                # No process has a handle on this.
                bucket.append(row + [None, None])

        # First, render internet connections.
        renderer.section("Active Internet connections")
        renderer.table_header(
            columns=[
                ("Proto", "proto", "14"),
                ("SAddr", "saddr", "30"),
                ("SPort", "sport", "8"),
                ("DAddr", "daddr", "30"),
                ("DPort", "dport", "5"),
                ("State", "state", "15"),
                ("Pid", "pid", "8"),
                ("Comm", "comm", "20"),
            ],
            sort=("pid", "proto", "saddr", "sport", "daddr", "dport", "state"),
        )

        for row in inet_socks:
            renderer.table_row(*row)

        # Render the UNIX sockets.
        renderer.section("Active UNIX domain sockets")
        renderer.table_header(
            columns=[
                ("Address", "address", "14"),
                ("Conn", "conn", "14"),
                ("Type", "type", "10"),
                ("Path", "path", "60"),
                ("Pid", "pid", "8"),
                ("Comm", "comm", "20"),
            ],
            sort=("pid", "address"),
        )

        for row in unix_socks:
            renderer.table_row(*row)



#!/usr/bin/env python

# Rekall
# Copyright (C) 2012 Michael Cohen <scudette@gmail.com>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

__author__ = "Michael Cohen <scudette@gmail.com>"

# pylint: disable=protected-access

import os
import pdb
import logging
import sys


from rekall import args
from rekall import config
from rekall import constants
from rekall import obj
from rekall import session

# Import and register the core plugins
from rekall import plugins  # pylint: disable=unused-import

try:
    from rekall import ipython_support
except ImportError:
    ipython_support = None


config.DeclareOption(
    "-r", "--run", default=None,
    help="Run this script before dropping into the interactive shell.")


config.DeclareOption(
    "-s", "--session_filename", default=None,
    help="If specified we save and restore the session from this filename.")


def IPython012Support(user_session):
    """Launch the ipython session for post 0.12 versions.

    Returns:
      False if we failed to use IPython. True if the session was run and exited.
    """
    if ipython_support:
        # This must be run here because the IPython shell messes with our user
        # namespace above (by adding its own help function).
        user_session._prepare_local_namespace()

        return ipython_support.Shell(user_session)


def NotebookSupport(user_session):
    engine = user_session.ipython_engine
    if not engine:
        return False

    if engine == "notebook":
        argv = ["notebook", "-c",
                "from rekall import interactive; "
                "interactive.ImportEnvironment();", "--autocall", "2"]
        import IPython

        IPython.start_ipython(argv=argv)
        return True
    else:
        raise RuntimeError("Unknown ipython mode %s" % engine)


def NativePythonSupport(user_session):
    """Launch the rekall session using the native python interpreter.

    Returns:
      False if we failed to use IPython. True if the session was run and exited.
    """
    # If the ipython shell is not available, we can use the native python shell.
    import code

    # Try to enable tab completion
    try:
        import rlcompleter, readline  # pylint: disable=W0612
        readline.parse_and_bind("tab: complete")
    except ImportError:
        pass

    # Prepare the session for running within the native python interpreter.
    user_session._prepare_local_namespace()
    code.interact(banner=constants.BANNER, local=user_session._locals)


def main(argv=None):
    # IPython notebook launches the IPython kernel by re-spawning the main
    # binary with its own command line args. This hack traps this and diverts
    # execution to IPython itself.
    if len(sys.argv) > 2 and sys.argv[1] == "-c":
        to_run = sys.argv[2]
        if ".kernelapp" in to_run:
            exec(to_run)
            return

    # New user interactive session (with extra bells and whistles).
    user_session = session.InteractiveSession()

    flags = args.parse_args(argv=argv, user_session=user_session)

    # Determine if an external script needs to be run first.
    if getattr(flags, "run", None):
        exec open(flags.run) in user_session._locals

    # Run a module and do not drop into the shell.
    if getattr(flags, "module", None):
        # Run the module
        try:
            # Explicitly disable our handling of the pager since we are not
            # running in interactive mode.
            user_session.RunPlugin(flags.module, flags=flags)
        except Exception as e:
            if getattr(flags, "debug", None):
                pdb.post_mortem(sys.exc_info()[2])
            else:
                logging.error("%s. Try --debug for more information." % e)

    else:
        # Interactive session, turn off object access logging since in
        # interactive mode, the user may use arbitrary object members.
        os.environ.pop(obj.ProfileLog.ENVIRONMENT_VAR, None)

        user_session.mode = "Interactive"

        # Try to launch the session using something.
        if user_session.state.ipython_engine == "notebook":
            ipython_support.NotebookSupport(user_session)
        else:
            _ = (IPython012Support(user_session) or
                 NativePythonSupport(user_session))

    # Right before we exit we check if we need to save the current session.
    if user_session.state.session_filename and (
        user_session.state.dirty or user_session.state.cache.dirty):
        user_session.SaveToFile(user_session.state.session_filename)


if __name__ == '__main__':
    main()



# Rekall Memory Forensics
# Copyright (C) 2014 Michael Cohen
# Copyright 2014 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""This module guesses the current profile using various heuristics."""

__author__ = "Michael Cohen <scudette@gmail.com>"

# pylint: disable=protected-access
import logging
import re

from rekall import config
from rekall import kb
from rekall import scan
from rekall.plugins.darwin import common as darwin_common
from rekall.plugins.linux import common as linux_common
from rekall.plugins.windows import common as win_common

OSX_NEEDLE = "Catfish \x00\x00"

PROFILE_STRINGS = [
    # Anything that looks like a RSDS record signature.
    "RSDS",

    # Found in every OS X image. See documentation for DarwinFindKASLR for
    # details.
    OSX_NEEDLE,

    # The Linux kernels we care about contain this.
    "Linux version ",
]


class ProfileScanner(scan.BaseScanner):
    checks = [("MultiStringFinderCheck", dict(needles=PROFILE_STRINGS))]


config.DeclareOption("--no_autodetect", default=False, action="store_true",
                     help="Should profiles be autodetected.")

config.DeclareOption("--autodetect_threshold", default=1.0, action="store",
                     help="Worst acceptable match for profile autodetection." +
                     " (Default 1.0)",
                     type=float)

class KernelASHook(kb.ParameterHook):
    """A ParameterHook for default_address_space.

    This will only be called if default_address_space is not set. We load the
    kernel address space, or load it if needed.
    """
    name = "default_address_space"

    def calculate(self):
        if not self.session.profile:
            self.session.GetParameter("profile")

        if self.session.kernel_address_space:
            return self.session.kernel_address_space

        return self.session.plugins.load_as().GetVirtualAddressSpace()


class ProfileHook(kb.ParameterHook):
    """If the profile is not specified, we guess it."""
    name = "profile"

    # Windows kernel pdb files.
    KERNEL_NAMES = win_common.KERNEL_NAMES

    LINUX_TEMPLATE = re.compile(
        r"Linux version (\d+\.\d+\.\d+[^ ]+)")


    def VerifyDarwinProfile(self, profile_name):
        try:
            # Try to load this profile from the repository.
            self.session.profile = self.session.LoadProfile(profile_name)
        except ValueError:
            return

        return self.ApplyFindDTB(darwin_common.DarwinFindDTB,
                                 self.session.profile)

    def VerifyLinuxProfile(self, profile_name):
        profile = self.session.LoadProfile(profile_name)
        if profile != None:
            return self.ApplyFindDTB(linux_common.LinuxFindDTB, profile)

    def VerifyWinProfile(self, profile_name):
        """Check that this profile works with this image.

        Currently we construct a valid DTB with this profile. This might have
        trouble distinguishing profiles which are fairly close (e.g. Win7
        versions).
        """
        logging.debug("Verifying profile %s", profile_name)

        try:
            # Try to load this profile from the repository.
            profile = self.session.LoadProfile(profile_name)
        except ValueError as e:
            logging.info("Error loading profile: %s" % e)
            return

        return self.ApplyFindDTB(win_common.WinFindDTB, profile)

    def ApplyFindDTB(self, find_dtb_cls, profile):
        """Verify profile by trying to use it to load the dtb.

        If this succeeds the profile is likely correct.
        """
        self.session.profile = profile

        find_dtb_plugin = find_dtb_cls(session=self.session)

        # Allow the dtb to be specified on the command line.
        dtb = self.session.GetParameter("dtb")
        if dtb:
            address_space = find_dtb_plugin.CreateAS(dtb)
            self.session.SetParameter("default_address_space", address_space)
            return profile

        for address_space in find_dtb_plugin.address_space_hits():
            # Might as well cache the results of this plugin so we dont need to
            # run it twice.
            self.session.kernel_address_space = address_space

            # Start off with a default address space of the kernel.
            with self.session as session:
                session.SetParameter("default_address_space", address_space)
                session.SetParameter("dtb", address_space.dtb)

            return profile

    def ScanProfiles(self):
        pe_profile = self.session.LoadProfile("pe")
        address_space = self.session.physical_address_space
        best_profile = None
        best_match = 0

        for hit in ProfileScanner(address_space=address_space,
                                  session=self.session).scan():

            # Try Windows by GUID:
            rsds = pe_profile.CV_RSDS_HEADER(offset=hit, vm=address_space)
            if (rsds.Signature.is_valid() and
                str(rsds.Filename) in self.KERNEL_NAMES):
                profile = self.VerifyWinProfile(
                    "nt/GUID/%s" % rsds.GUID_AGE)

                if profile:
                    logging.info(
                        "Detected %s with GUID %s", rsds.Filename,
                        rsds.GUID_AGE)

                    best_profile = profile
                    best_match = 1

            # Try OS X by profile similarity:
            elif address_space.read(hit, len(OSX_NEEDLE)) == OSX_NEEDLE:
                # To work around KASLR, we have an index of known symbols'
                # offsets relative to the Catfish string, along with the data we
                # expect to find at those offsets. Profile similarity is the
                # percentage of these symbols that match as expected.
                #
                # Ideally, we'd like a 100% match, but in case we don't have the
                # exact profile, we'll make do with anything higher than 0% that
                # can resolve the DTB.
                logging.debug("Hit for Darwin at 0x%x", hit)
                index = self.session.LoadProfile("OSX/index")
                for profile_name, match in index.LookupIndex(
                    image_base=hit,
                    address_space=self.session.physical_address_space):
                    profile = self.VerifyDarwinProfile(profile_name)
                    if profile:
                        if match > best_match:
                            logging.info(
                                "New best match: %s (%.0f%% match)",
                                profile_name, match * 100)
                            best_profile, best_match = profile, match
                            self.session.SetParameter("catfish_offset", hit)

                            if match == 1.0:
                                break

            # Try Linux by version string:
            else:
                guess = address_space.read(hit-100, 300)

                m = self.LINUX_TEMPLATE.search(guess)
                if m:
                    # Try to guess the distribution.
                    distribution = "LinuxGeneric"
                    if "Ubuntu" in guess:
                        distribution = "Ubuntu"

                    profile_name = "%s/%s" % (distribution, m.group(1))
                    profile = self.VerifyLinuxProfile(profile_name)
                    if profile:
                        logging.info(
                            "Detected %s: %s", profile_name, m.group(0))

                        best_profile = profile
                        best_match = 1

            if best_match == 1.0:
                # If we have an exact match we can stop scanning.
                break

        threshold = self.session.GetParameter("autodetect_threshold")
        if best_match == 0:
            logging.error(
                "No profiles match this image. Try specifying manually.")

        elif best_match < threshold:
            logging.error(
                "Best match for profile is %s with %.0f%%, which is too low " +
                "for given threshold of %.0f%%. Try lowering " +
                "--autodetect-threshold.",
                best_profile.name,
                best_match * 100,
                threshold * 100)

        else:
            logging.info(
                "Profile %s matched with %.0f%% confidence.",
                best_profile.name,
                best_match * 100)

            return best_profile

    def calculate(self):
        """Try to find the correct profile by scanning for PDB files."""
        if not self.session.physical_address_space:
            # Try to load the physical_address_space so we can scan it.
            if not self.session.plugins.load_as().GetPhysicalAddressSpace():
                # No physical address space - nothing to do here.
                return

        # Only do something only if we are allowed to autodetect profiles.
        if not self.session.GetParameter("no_autodetect"):
            return self.ScanProfiles()
